import os
from collections import namedtuple

class WinCCOAProject(object):
    
    class Prog(namedtuple('Prog', 'i name start_mode sec_kill restart_count reset_min command_line')):
        @classmethod
        def fromDiskLine(Prog, lineNumber, line):
            return Prog(lineNumber-4, *[s.strip() for s in line.split('|')])
        
        def toDiskLine(self):
            return '%s| %s | %s | %s | %s |%s\n' % (
                self.name.ljust(17), self.start_mode.rjust(6),
                self.sec_kill.rjust(7), self.restart_count.rjust(8),
                self.reset_min.rjust(8), self.command_line
            )
    
    def __init__(self, projectPath):
        self.projectPath = projectPath
        self.projectName = os.path.basename(self.projectPath)
        
    def getConsoleManagers(self, progs_file='progs'):
        with open(os.path.join(self.projectPath, 'config', progs_file), 'r') as progsFile:
            lineNumber = 0
            for line in progsFile:
                if lineNumber > 3:
                    yield self.Prog.fromDiskLine(lineNumber, line)
                lineNumber += 1
    
    def filterProgs(self, progs=None, **kwargs):
        if not progs:
            progs = self.getConsoleManagers()
        for prog in progs:
            #if len(kwargs.viewitems() & prog._asdict().viewitems()) == len(kwargs): # Only python 2.7
            if len(set(kwargs.items()) & set(prog._asdict().items())) == len(kwargs):
                yield prog
            
    def updateConsoleManagers(self, *progs):
        with open(os.path.join(self.projectPath, 'config', 'progs'), 'r') as progsFile:
            lines = progsFile.readlines()
            
        for prog in progs:
            lines[prog.i+4] = prog.toDiskLine()
            
        with open(os.path.join(self.projectPath, 'config', 'progs'), 'w') as progsFile:
            progsFile.writelines(lines)
            
    def __repr__(self):
        return 'WinCCOAProject("%s")' % self.projectPath
