#uses "fwInstallation.ctl"

// Need option configDB
// Need option configDBUser
// Need option configDBEncPwd
int PVSSBootstrapperDB_prepareConnection(mapping &options) {
  int err;
  
  err = fwInstallationDB_openDBConnection(options["configDB"], options["configDBUser"], options["configDBEncPwd"], "");
  if(err < 0) {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapperDB_prepareConnection() -> Could not connect to the database " + options["configDB"] + " with user " + options["configDBUser"]);
    return -1;
  }
  
  g_PVSSBootstrapperUseDB = true;
  
  return 0;
}

// Need option debug
// Set option agentRequestID
// Set option agentProjectID
// Set option agentAction
// Set option fwInstZipPath
int PVSSBootstrapperDB_getLastRequest(mapping &options/* string hostname, int &requestID, int &projectID, string &action, string &fwInstallPath*/) {
  string sql;
  dyn_dyn_mixed aRecords;
  
  string hostname = fwInstallation_getHostname();
  
  dynClear(aRecords);  
  hostname = strtoupper(hostname);
  
  dyn_mixed var;
  var[1] = hostname;
  
  sql = 
  "SELECT r.id, r.project_id, r.action, r.fwInstall_path " +
  "  FROM FW_SYS_STAT_AGENT_REQUEST r, " +
  "       FW_SYS_STAT_COMPUTER c, " +
  "       FW_SYS_STAT_PVSS_PROJECT p " +
  "  WHERE r.executed_on is null " +
  "    AND c.hostname = :1 " +
  "    AND c.valid_until is null " +
  "    AND (p.computer_id = c.id OR p.redu_computer_id = c.id) " +
  "    AND r.project_id = p.id " +
  "    AND rownum = 1 " + // This is Oracle dependant
  "  ORDER BY r.requested_on DESC";
  
      
  if(g_fwInstallationSqlDebug && options["debug"])
    DebugN(sql, var);

  if(fwInstallationDB_executeQuery(sql, var, aRecords)) {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapperDB_getLastRequest() -> Could not execute the following SQL query: " + sql);
    return -1;
  }
  
  if(dynlen(aRecords) > 0) {
    options["agentRequestID"] = aRecords[1][1];
    options["agentProjectID"] = aRecords[1][2];
    options["agentAction"] = aRecords[1][3];
    options["fwInstZipPath"] = aRecords[1][4];
  }else
    options["agentRequestID"] = -1;

  return 0;
}

// Need option debug
// Set option projectName
// Set option installDir
// Set option systemNumber
// Set option systemName
// Set option pmonPort
// Set option dataPort
// Set option eventPort
// Set option distPort
int PVSSBootstrapperDB_getConfigFromDBSysInfo(mapping &options) {
  int err;
  dyn_mixed projectProperties;
  dyn_mixed connectedSystemsInfo;
  
  // Get project props
  err = _PVSSBootstrapperDB_getProjectPropertiesByID(options, projectProperties);
  if(err < 0)
    return err;
  
  if(options["debug"])
    DebugN("Project info", projectProperties);
    
  // Get dist info
  err = fwInstallationDB_getSystemConnectivity(projectProperties[FW_INSTALLATION_DB_PROJECT_SYSTEM_NAME], connectedSystemsInfo, true);
  if(err < 0) {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapperDB_getConfigFromDBSysInfo() -> Error while getting dist info with fwInstallationDB_getSystemConnectivity().");
    return err;
  }
  
  if(options["debug"])
    DebugN("Dist info", connectedSystemsInfo);

  // Populate all parameters needed for the project creation.
  
  options["projectName"] = projectProperties[FW_INSTALLATION_DB_PROJECT_NAME]; // Projectname of the new project
  
  options["installDir"] = _PVSSBootstrapperDB_handleTargetPath(projectProperties[FW_INSTALLATION_DB_PROJECT_DIR]); // Targetdirectory of the new project
  
  options["systemNumber"] = projectProperties[FW_INSTALLATION_DB_PROJECT_SYSTEM_NUMBER]; // System number of the new project
  options["systemName"] = projectProperties[FW_INSTALLATION_DB_PROJECT_SYSTEM_NAME]; // System name of the new project
  
  // dyn_string reduHosts = makeDynString(); // TODO: reduHosts({}): Hostname (Data,Event) of the new project // [1]Host1 ... Reduhost1 // [2]Host2 ... Reduhost2
  
  // dyn_string subProjs = makeDynString(); // Subprojects to add
  
  /* dyn_string distPeers = makeDynString(); // list of connected distributed systems // "Disthost1[$Disthost1redu[:Distport]]" SysNr1 // "Disthost2[$Disthost2redu[:Distport]]" SysNr2
  for(int i=1 ; i <= dynlen(connectedSystemsInfo) ; i++)
    dynAppend(distPeers,
              "\"" + connectedSystemsInfo[i][FW_INSTALLATION_DB_SYSTEM_NAME] + "\""
              + connectedSystemsInfo[i][FW_INSTALLATION_DB_SYSTEM_NUMBER]); // TODO: Support for redu in dist (possible with current  fwInstallationDB_getSystemConnectivity ?)
  */
  
  options["pmonPort"] = projectProperties[FW_INSTALLATION_DB_PROJECT_PMON_PORT]; // Port of the pmon of new project
  options["dataPort"] = projectProperties[FW_INSTALLATION_DB_PROJECT_DATA]; // Port of the data of new project
  options["eventPort"] = projectProperties[FW_INSTALLATION_DB_PROJECT_EVENT]; // Port of the event of new project
  options["distPort"] = projectProperties[FW_INSTALLATION_DB_PROJECT_DIST]; // Port of the dist of new project
  
  // TODO: PMON user/pwd (has to support the "N/A" value in the DB)
  
  return 0;
}

string _PVSSBootstrapperDB_handleTargetPath(string pathFromDB) {
  dyn_string pathSplitted = strsplit(paPathSlashes(pathFromDB), "/");
  string path = "";
  for(int i=1 ; i < dynlen(pathSplitted) ; i++) {
    if(pathSplitted[i] != "")
      path += "/" + pathSplitted[i];
  }
  
  // Remove the "/" at the root
  if(_WIN32)
    path = substr(path, 1);
  
  return path;
}

// Need option agentProjectID
// Need option debug
int _PVSSBootstrapperDB_getProjectPropertiesByID(mapping &options, dyn_mixed &projectProperties) {
  string sql;
  dyn_dyn_mixed aRecords;
  
  dynClear(aRecords);  
  dynClear(projectProperties);
  
  dyn_mixed var;
  var[1] = options["agentProjectID"];  
  
  sql = 
  "SELECT p.project_name, c.hostname, p.project_dir, s.system_name, s.system_number, " +
  "       p.pmon_port, p.pmon_username, p.pmon_password, p.fw_inst_tool_version, " +
  "       p.centrally_managed, v.version_name, s.data_port, s.event_port, s.dist_port, " +
  "       v.os, p.is_project_ok, p.is_pvss_ok, p.is_host_ok, p.is_path_ok, p.is_manager_ok, " +
  "       p.is_group_ok, p.is_component_ok, p.is_ext_process_ok, p.last_time_checked, " +
  "       p.system_overview, p.upgrade, NULL, NULL, p.delete_files " + 
  "  FROM fw_sys_stat_pvss_project p, " +
  "       fw_sys_stat_pvss_system s, " + 
  "       fw_sys_stat_computer c, " + 
  "       fw_sys_stat_pvss_version v, " +
  "       fw_sys_stat_pvss_base_version b " +
  "  WHERE p.id = :1 " +
  "    AND c.valid_until is null " +
  "    AND (p.computer_id = c.id OR p.redu_computer_id = c.id) " +
  "    AND p.system_id = s.id " +
  "    AND b.id = p.pvss_base_id " +
  "    AND v.id = b.pvss_version_id ";
  
      
  if(g_fwInstallationSqlDebug && options["debug"])
    DebugN(sql, var);

  if(fwInstallationDB_executeQuery(sql, var, aRecords)) {
    PVSSBootstrapper_log("ERROR", "_PVSSBootstrapperDB_getProjectPropertiesByID() -> Could not execute the following SQL query: " + sql);
    return -1;
  }
  
  if(dynlen(aRecords) > 0) {
    projectProperties = aRecords[1];
    //resolve redundant pair:
    string reduHost;
    string project = projectProperties[FW_INSTALLATION_DB_PROJECT_NAME];
    string hostname = projectProperties[FW_INSTALLATION_DB_PROJECT_HOST];
    fwInstallationDB_getReduPair(hostname, project, reduHost);
    projectProperties[FW_INSTALLATION_DB_PROJECT_REDU_HOST] = reduHost;
  }else{
    PVSSBootstrapper_log("ERROR", "_PVSSBootstrapperDB_getProjectPropertiesByID() -> No project found by the ID " + projectID);
    return -1;
  }

  return 0;
}

// Need option agentRequestID
// Need option debug
int _PVSSBootstrapperDB_setExecutedOnNow(mapping &options) {
  string sql;
  
  PVSSBootstrapper_log("INFO", "Setting the executed_on time for request " + options["agentRequestID"]);

  dyn_mixed record;
  record[1] = options["agentRequestID"];

  sql = "UPDATE fw_sys_stat_agent_request SET executed_on=SYSDATE WHERE id = :1";
  
      
  if(g_fwInstallationSqlDebug && options["debug"])
    DebugN(sql, var);
         
  if(fwInstallationDB_execute(sql, record)) {
    PVSSBootstrapper_log("ERROR", "ERROR: _PVSSBootstrapperDB_setExecutedOnNow() -> Could not execute the following SQL: " + sql);
    return -1;
  }

  return 0;  
}

int _PVSSBootstrapperDB_log(int requestID, string logLevel, string logText) {
  string sql;
  
  if(strlen(logText) > 4000)
    logText = substr(logText, 0, 4000);
  
  if(strlen(logLevel) > 10)
    logLevel = substr(logLevel, 0, 10);

  dyn_mixed record;
  record[1] = requestID;
  record[2] = logLevel;
  record[3] = logText;

  sql = "INSERT INTO fw_sys_stat_agent_log(log_time, request_id, log_level, log_text) " +
        "  VALUES (SYSTIMESTAMP, :1 ,:2, :3)";
         
  if(fwInstallationDB_execute(sql, record)) {
    _PVSSBootstrapper_throw("ERROR: _PVSSBootstrapperDB_log() -> Could not execute the following SQL: " + sql);
    return -1;
  }

  return 0;  
}
