#uses "fwInstallation.ctl"

int PVSSBootstrapper_installComponents(mapping &options, bool &needRestart) {
  int err;
  
  needRestart = false;
  
  if(dynlen(options["installComponents"]) == 0) // Nothing to do
    return 0;
  
  err = _PVSSBootstrapper_initFwInstallation();
  if(err < 0)
    return err;

  err = _PVSSBootstrapper_checkComponentsConsistency(options);
  if(err < 0)
    return err;

  // Init fwInstallation log
  //fwInstallation_resetLog();
  //fwInstallation_writeToMainLog(formatTime("[%Y-%m-%d_%H:%M:%S] ", getCurrentTime()) + "Starting installation from PVSS Bootstrapper."); 

  dyn_string availableComps = makeDynString();
  dyn_string availableCompsFullPath = makeDynString();
  dyn_string compSources = options["componentSources"];
  dyn_string compToInstall = options["installComponents"];
  
  for(int i = 1 ; i <= dynlen(compSources) ; i++) {
    dyn_string candidates = getFileNames(compSources[i], "*.xml", FILTER_FILES);
    for(int j=1 ; j <= dynlen(candidates) ; j++) {
      dynAppend(availableComps, candidates[j]);
      dynAppend(availableCompsFullPath, compSources[i] + "/" + candidates[j]);
    }
  }
  
  // Get all components associated with their source directory
  dyn_string componentNames, componentVersions, componentFiles, componentFilesInOrder;
  for(int i=1 ; i <= dynlen(compToInstall) ; i++) {
    int id = dynContains(availableComps, compToInstall[i]);
    if(id <= 0) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_installComponents() -> Inconsistent list of components to install. Component " + compToInstall[i] + " is not in the available sources");
      return -1;
    }
    string componentPath = availableCompsFullPath[id];
    string componentName = fwInstallation_getComponentName(componentPath);
    dynAppend(componentNames, componentName);
    dynAppend(componentFiles, componentPath);
  }
	  
  // Ensure there is a slash at the end of the source paths ("caveat" from fwInstallation, it might be that it is somewhere needed to have "//" afterwards).
  /*dyn_string compSources = options["componentSource"];
  for(int i=1 ; i <= dynlen(compSources) ; i++) {
    if(substr(compSources[i], strlen(compSources[i])-1, 1) != "/")
      compSources[i] += "/";
  }*/
  
  // Get all components associated with their source directory
  /*dyn_string componentNames, componentVersions, componentFiles, componentFilesInOrder;
  dyn_string compToInstall = options["installComponent"];
  for(int i=1 ; i <= dynlen(compToInstall) ; i++) {
    int sourceIndex;
    string componentName;
    if(sscanf(compToInstall[i], "[%d] %s", sourceIndex, componentName) != 2) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_installComponents() -> Malformatted component name: " + compToInstall[i]);
      return -1;
    }
  
    dynAppend(componentNames, componentName);
    dynAppend(componentFiles, compSources[sourceIndex] + componentName);
  }*/
  
  // Order the list of components to install (dependency resolution)
  fwInstallation_putComponentsInOrder_Install(componentNames, componentVersions, componentFiles, componentFilesInOrder);
  
  // Backup project config file before starting the installation
  err = fwInstallation_backupProjectConfigFile();
  if(err != 0) {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_installComponents()/fwInstallation -> Unable to backup project config file.");
    return -1;
  }
  
  // Install the components in order
  for(int i=1 ; i<= dynlen(componentFilesInOrder) ; i++) {
    err = _PVSSBootstrapper_installComponent(componentFilesInOrder[i], needRestart);
    if(err < 0)
      return err;
  }
  
  // Update the System Configuration DB if required
  if(fwInstallationDB_getUseDB()) {
    err = fwInstallationDB_connect();
    if(err != 0) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_installComponents()/fwInstallation -> Failed to connect to the system Configuration DB after component installation.");
      return -1;
    }
    
    int shouldRestart;
    err = fwInstallationDBAgent_synchronize(shouldRestart);
    if(err != 0) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_installComponents()/fwInstallation -> Failed to update the System Configuration DB after component installation.");
      return -1;
    }
    if(shouldRestart == 1)
      needRestart = true;
      
    //err = fwInstallationDB_storeInstallationLog();
    //if(err != 0) {
    //  PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_installComponents()/fwInstallation -> Failed to store logs into the System Configuration DB.");
    //  return -1;
    //}
  }
  
  // If don't need restart, execute post installation scripts by restarting the fwScripts control manager
  if(!needRestart) {
    err = fwInstallationManager_command("START", fwInstallation_getWCCOAExecutable("ctrl"), "-f fwScripts.lst");
    if(err != 0) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_installComponents()/fwInstallation -> Unable to restart the fwScripts control manager. Post installation scripts could not be executed.");
      return -1;
    }
  }
  
	//fwInstallation_writeToMainLog(formatTime("[%Y-%m-%d_%H:%M:%S] ", getCurrentTime()) + "Finishing installation.");
	//string tempString = options["componentInstallDir"] + "/log/fwInstallation_" + formatTime("%Y%m%d_%H%M%S", getCurrentTime()) + ".log";
	//fwInstallation_writeToMainLog(formatTime("[%Y-%m-%d_%H:%M:%S] ", getCurrentTime()) + "Please find the log file for this installation operation in: " + tempString);
  
  return 0;
}

int _PVSSBootstrapper_initFwInstallation() {
  int err;
  
  // First, initialize fwInstallation
	err = fwInstallation_init();
  if(err != 0) {
    PVSSBootstrapper_log("ERROR", "_PVSSBootstrapper_initFwInstallation()/fwInstallation -> Error while initialising fwInstallation.");
    return -1;
  }
  
  return 0;
}

int PVSSBootstrapper_setComponentInstallDir(mapping &options) {
  int err;
  
	err = _PVSSBootstrapper_initFwInstallation();
	if(err < 0)
	  return err;
  
  string installDir = options["componentInstallDir"];
  
  if(installDir == "")
    return 0;
  
  string fwInstallationDP = fwInstallation_getInstallationDp();
  
  // Define the component installation directory
  if((substr(installDir, 0, 1) != "/") && (substr(installDir, 2, 1) != "\\")) // Linux root path "/" && Windows drive root path "X:\"
    installDir = dirName(getPath(SCRIPTS_REL_PATH)) + installDir + "/";
  if(!isdir(installDir))
    mkdir(installDir, "755");  
  
  // Set the install directory DP
  if(!dpExists(fwInstallationDP + ".installationDirectoryPath")) {
    dpCreate(fwInstallationDP , "_FwInstallationInformation");
  }
  dpSet(fwInstallation_getInstallationDp() + ".installationDirectoryPath", installDir);
  
  // Set the install directory as a project path
  err = fwInstallation_addProjPath(installDir, 999);
  if(err != 0) {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_setComponentInstallDir()/fwInstallation -> Error while adding the project path: " + installDir);
    return err;
  }
  
  PVSSBootstrapper_log("INFO", "Setting component install directory to " + installDir);
  
  return 0;
}

int _PVSSBootstrapper_checkComponentsConsistency (mapping &options) {
  dyn_string availableComps = makeDynString();
  dyn_string compSources = options["componentSources"];
  dyn_string compToInstall = options["installComponents"];
  
  for(int i = 1 ; i <= dynlen(compSources) ; i++) {
    dyn_string candidates = getFileNames(compSources[i], "*.xml", FILTER_FILES);
    for(int j=1 ; j <= dynlen(candidates) ; j++)
      dynAppend(availableComps, candidates[j]);
  }
  
  for(int i=1 ; i <= dynlen(compToInstall) ; i++) {
    if(!dynContains(availableComps, compToInstall[i])) {
      PVSSBootstrapper_log("ERROR", "_PVSSBootstrapper_checkComponentsConsistency() -> Inconsistent list of components to install. Component " + compToInstall[i] + " is not in the available sources");
      return -1;
    }
  }
  
  return 0;
}

int _PVSSBootstrapper_checkAndStopManagers(string type) {
  if(fwInstallationManager_shallStopManagersOfType(type)) {
    if(fwInstallationManager_stopAllOfTypes(makeDynString(type)) != 0) {
	    PVSSBootstrapper_log("ERROR", "_PVSSBootstrapper_checkAndStopManagers()/fwInstallation -> Unable to stop managers of type " + type);
	    return -1;
    }
  }
  return 0;
}

int _PVSSBootstrapper_installComponent(string componentFile, bool &needRestart) {
  int err;
  bool isItSubComponent;
  string componentName;
  bool componentInstalled;
  string dontRestartProject;
  string subPath = "";
  bool forceInstallRequired = true;
  bool forceOverwriteFiles = true;
  bool isSilent = true;
  bool installSubComponents = true;
  
  string source = dirName(componentFile);
  
  err = fwInstallation_isSubComponent(componentFile, isItSubComponent);
  if(err != 0) {
    PVSSBootstrapper_log("ERROR", "_PVSSBootstrapper_installComponent()/fwInstallation -> Error while determining if component is sub-component from " + componentFile);
    return -1;
  }
  
  PVSSBootstrapper_log("INFO", "Installing component from " + componentFile);
  
  err = fwInstallation_installComponent(componentFile, source, isItSubComponent,
                                        componentName, componentInstalled,
                                        dontRestartProject, subPath, forceInstallRequired,
                                        forceOverwriteFiles, isSilent, installSubComponents);
  if((err != 0) || !componentInstalled) {
    PVSSBootstrapper_log("ERROR", "_PVSSBootstrapper_installComponent()/fwInstallation -> Error while installing component from " + componentFile);
    return -1;
  }
  
  PVSSBootstrapper_log("INFO", "Installed component " + componentName);
  
  if(fwInstallation_isProjectRestartAfterInitPending() != 0)
    needRestart = true;
  
  return 0;
}

int PVSSBootstrapper_restartProject() {
  int err;
  
  PVSSBootstrapper_log("INFO", "Restarting the project");
  
  err = fwInstallation_forceProjectRestart();
  if(err != 0) {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_restartProject()/fwInstallation -> Cannot restart the project.");
    return err;
  }
  
  return 0;
}

