string _normalConfigFile = "config.PVSSBootstrapper";

int PVSSBootstrapper_getConfig(mapping &options) {
  _normalConfigFile = getPath(CONFIG_REL_PATH, _normalConfigFile);
  
  DebugN(_normalConfigFile);

  options["configFile"] = _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "configFile", "");
  
  // Get the source
  if(_PVSSBootstrapper_getAndCheckValue(options, "configSource", "getConfig") < 0)
    return -1;
    
  // Get the config entries valid for any sources
  options["debug"] = (bool) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "debug", "false", options["configFile"]);
  options["executionMode"] = _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "executionMode", "OneShot", options["configFile"]);
  options["agentSleep"] = (int) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "agentSleep", "5", options["configFile"]);

  if(strtoupper(options["configSource"]) == "CONFDBSYSINFO") { // Get the source when its from the DB
    if(_PVSSBootstrapper_getConfigForDBSysInfo(options) < 0)
      return -1;
  }else if(strtoupper(options["configSource"]) == "CONFIGFILE") { // Get the source when its from the config file
    if(_PVSSBootstrapper_getConfigFromFile(options) < 0)
      return -1;
  }
  
  options["postProcessScript"] = _PVSSBootstrapper_getListFromConfigFile("PVSSBootstrapper", "postProcessScript", options["configFile"]);
  
  return 0;
}

// Loading special options for DB sys info.
int _PVSSBootstrapper_getConfigForDBSysInfo(mapping &options) {
  if(_PVSSBootstrapper_getAndCheckValue(options, "configDB", "getConfigFromDBSysInfo") < 0)
    return -1;
    
  if(_PVSSBootstrapper_getAndCheckValue(options, "configDBUser", "getConfigFromDBSysInfo") < 0)
    return -1;
  
  if(_PVSSBootstrapper_getAndCheckValue(options, "configDBEncPwd", "getConfigFromDBSysInfo") < 0)
    return -1;
    
  return 0;
}

// Loading options for a config file.
int _PVSSBootstrapper_getConfigFromFile(mapping &options) {
  if(_PVSSBootstrapper_getAndCheckValue(options, "function", "getConfigFromFile") < 0)
    return -1;
    
  if(strtoupper(options["function"]) == "STARTPROJECT") {
    options["projectPath"] = _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "projectPath", "", options["configFile"]);
    options["projectName"] = _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "projectName", "", options["configFile"]);
    if((options["projectPath"] == "") && (options["projectName"] == "")) {
      PVSSBootstrapper_log("ERROR", "_PVSSBootstrapper_getConfigFromFile() -> projectName or projectPath must be specified in the config file");
      return -1;
    }
    
    options["restart"] = (bool) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "restart", "false", options["configFile"]);
    
    options["user"] = _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "user", "", options["configFile"]);
    
    options["password"] = _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "password", "", options["configFile"]);
    
    options["openConsole"] = (bool) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "openConsole", "true", options["configFile"]);
    
    options["openLogViewer"] = (bool) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "openLogViewer", "true", options["configFile"]);
    
    options["dontRegister"] = (bool) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "dontRegister", "false", options["configFile"]);
    
    options["dontWait"] = (bool) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "dontWait", "false", options["configFile"]);
    
    options["onlyPmon"] = (bool) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "onlyPmon", "false", options["configFile"]);
    
  }else if(strtoupper(options["function"]) == "STOPPROJECT") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectName", "getConfigFromFile") < 0)
      return -1;
    
  }else if(strtoupper(options["function"]) == "STOPALLPROJECTS") {
    // No option
    
  }else if(strtoupper(options["function"]) == "ISPROJECTRUNNING") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectName", "getConfigFromFile") < 0)
      return -1;
    
  }else if(strtoupper(options["function"]) == "REGISTERPROJECT") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectPath", "getConfigFromFile") < 0)
      return -1;
      
    options["pmonPort"] = (int) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "pmonPort", "0", options["configFile"]);
    
  }else if(strtoupper(options["function"]) == "UNREGISTERPROJECT") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectName", "getConfigFromFile") < 0)
      return -1;
    
    options["dontStop"] = (bool) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "dontStop", "false", options["configFile"]);
    
  }else if(strtoupper(options["function"]) == "DELETEPROJECT") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectName", "getConfigFromFile") < 0)
      return -1;
    
    options["dontStop"] = (bool) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "dontStop", "false", options["configFile"]);
    
  }else if(strtoupper(options["function"]) == "ISPROJECTREGISTERED") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectName", "getConfigFromFile") < 0)
      return -1;
    
  }else if(strtoupper(options["function"]) == "CREATEPROJECT") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "installPath", "getConfigFromFile") < 0)
      return -1;
      
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectName", "getConfigFromFile") < 0)
      return -1;
      
    options["systemNumber"] = (int) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "systemNumber", "1", options["configFile"]);
      
    options["systemName"] = _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "systemName", "dist_" + options["systemNumber"], options["configFile"]);

    // TODO: reduHosts

    // TODO: distPeers
      
    options["overwrite"] = (bool) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "overwrite", "false", options["configFile"]);
      
    options["pmonPort"] = (int) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "pmonPort", "0", options["configFile"]);
      
    options["dataPort"] = (int) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "dataPort", "0", options["configFile"]);
      
    options["eventPort"] = (int) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "eventPort", "0", options["configFile"]);
    
    // TODO: reduPort

    // TODO: splitPort
      
    options["distPort"] = (int) _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", "distPort", "0", options["configFile"]);
    
    // TODO: projectUser

    // TODO: projectPassword
  }else if(strtoupper(options["function"]) == "GETPROJECTPATH") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectName", "getConfigFromFile") < 0)
      return -1;
    
  }else if(strtoupper(options["function"]) == "LISTPROJECTS") {
    // No option
    
  }else if(strtoupper(options["function"]) == "DEPLOYZIP") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectName", "getConfigFromFile") < 0)
      return -1;
      
    if(_PVSSBootstrapper_getAndCheckValue(options, "zipFile", "getConfigFromFile") < 0)
      return -1;
    
  }else if(strtoupper(options["function"]) == "SETCOMPINSTDIR") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectName", "getConfigFromFile") < 0)
      return -1;
      
    if(_PVSSBootstrapper_getAndCheckValue(options, "componentInstallDir", "getConfigFromFile") < 0)
      return -1;
    
  }else if(strtoupper(options["function"]) == "INSTALLCOMPONENT") {
    if(_PVSSBootstrapper_getAndCheckValue(options, "projectName", "getConfigFromFile") < 0)
      return -1;
      
    options["componentSources"] = _PVSSBootstrapper_getListFromConfigFile("PVSSBootstrapper", "componentSources", options["configFile"]);
    if(dynlen(options["componentSources"]) < 1) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_getConfigFromFile() -> componentSources must be specified in the config file");
      return -1;
    }
      
    options["installComponents"] = _PVSSBootstrapper_getListFromConfigFile("PVSSBootstrapper", "installComponents", options["configFile"]);
    if(dynlen(options["installComponents"]) < 1) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_getConfigFromFile() -> installComponents must be specified in the config file");
      return -1;
    }
    
  }else if(strtoupper(options["function"]) == "PING") {
    // Ping function is for test purpose
    
  }else{
    PVSSBootstrapper_log("ERROR", "_PVSSBootstrapper_getConfigFromFile() -> Unknown function: " + options["function"]);
    return -1;
  }
  
  return 0;
}

// For required config entries
int _PVSSBootstrapper_getAndCheckValue(mapping &options, string key, string function) {
  options[key] = _PVSSBootstrapper_getStringFromConfigFile("PVSSBootstrapper", key, "", options["configFile"]);

  if(options[key] == "") {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_" + function + "() -> " + key + " must be specified in the config file");
    return -1;
  }else
    return 0;
}

// Load from the optional config file first if possible.
// If got default value, then try in the _normalConfigFile.
string _PVSSBootstrapper_getStringFromConfigFile(string section, string key, string defaultValue, string optionalConfigFile="", bool specialProcessing=false) {
  string value;
  
  bool isDefault=false;
  if(optionalConfigFile != "")
    value = paCfgReadValueDflt(optionalConfigFile, section, key, defaultValue, isDefault);
  
  if(isDefault || (optionalConfigFile == ""))
    value = paCfgReadValueDflt(_normalConfigFile, section, key, defaultValue, isDefault);
    
  if(specialProcessing)
    strreplace(value, "{date}", formatTime("%y%m%d", getCurrentTime()));
    
  return value;
}

// Load list from the optional config file first if possible.
// If got nothing value, then try in the _normalConfigFile.
dyn_string _PVSSBootstrapper_getListFromConfigFile(string section, string key, string optionalConfigFile="") {
  int err;
  dyn_string value;
  
  if(optionalConfigFile != "")
    err = paCfgReadValueList(optionalConfigFile, section, key, value);
  
  if((err < 0) || (optionalConfigFile == ""))
    err = paCfgReadValueList(_normalConfigFile, section, key, value);
    
  return value;
}

int PVSSBootstrapper_setStringInConfigFile(string section, string key, string value) {
  int err;
  
  err = paCfgInsertValue(_normalConfigFile, section, key, value);
  if(err < 0) {
    PVSSBootstrapper_log("ERROR", "_PVSSBootstrapper_setStringInConfigFile() -> Unable to insert value in config file " + _normalConfigFile);
    return -1;
  }
  
  return 0;
}

/*
dyn_string _PVSSBootstrapper_getListFromConfigFile(string configFile, string section, string key) {
  dyn_string values;
  
  if(configFile != "")
    paCfgReadValueList(configFile, section, key, values);
  else
    values = makeDynString();
  
  return values;
}
*/
