int g_agentRequestId = -1; // Used to ease logging

// Need option executionMode
// Could need option configSource
int PVSSBootstrapper_execute (mapping &options) {
  if(strtoupper(options["executionMode"]) == "ONESHOT")
    return PVSSBootstrapper_executeOneShot(options);
  else if(strtoupper(options["executionMode"]) == "AGENT") {
    if(strtoupper(options["configSource"]) == "CONFDBSYSINFO")
      return PVSSBootstrapper_executeAgent(options);
    else{
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_execute() -> Agent mode only possible with configSource == ConfDBSysInfo");
      return -1;
    }
  }else{
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_execute() -> Unknown execution mode: " + options["executionMode"]);
    return -1;
  }
}

// Need option agentSleep
int PVSSBootstrapper_executeAgent (mapping &options) {
  int err;
  
  err = PVSSBootstrapperDB_prepareConnection(options);
  if(err < 0)
    return err;
  
  PVSSBootstrapper_log("INFO", "Now start waiting for request in the DB. Delay: " + options["agentSleep"] + "s");
  while(1) {
    delay(options["agentSleep"]); 
    
    err = PVSSBootstrapperDB_getLastRequest(options);
    if(err < 0)
      return err;
      
    if(options["agentRequestID"] == -1) // No request found
      continue;
      
    g_agentRequestId = options["agentRequestID"];
    
    string action = strtoupper(options["agentAction"]);
    if((action == "CREATE") || (action == "RECREATE")) {
      PVSSBootstrapper_log("INFO", "Executing action " + action + ". Request ID: " + g_agentRequestId + ". Project ID: " + options["agentProjectID"]);
      
      // Set all required options
      options["procedure"] = "ProjectCreation";
      options["deleteFirstIfExists"] = (action == "CREATE") ? false : true;
      err = PVSSBootstrapperDB_getConfigFromDBSysInfo(options);
      if(err < 0) {
        PVSSBootstrapper_log("ERROR", "Agent has to stop");
        g_agentRequestId = -1;
        return err;
      }
      
      err = PVSSBootstrapper_doFunction(options);
      if(err < 0) {
        PVSSBootstrapper_log("ERROR", "Agent has to stop");
        g_agentRequestId = -1;
        return err;
      }
      
      // Clean options
      mappingRemove(options, "procedure");
      mappingRemove(options, "deleteFirstIfExists");
      mappingRemove(options, "projectName");
      mappingRemove(options, "installDir");
      mappingRemove(options, "systemNumber");
      mappingRemove(options, "systemName");
      mappingRemove(options, "pmonPort");
      mappingRemove(options, "dataPort");
      mappingRemove(options, "eventPort");
      mappingRemove(options, "distPort");
    }else // If implement later action for ProjectStarter, don't forget to clean option projectName (because of function registerProject)
      PVSSBootstrapper_log("ERROR", "Unknown action: " + action);
    
    err = _PVSSBootstrapperDB_setExecutedOnNow(options);
    if(err < 0) {
      PVSSBootstrapper_log("ERROR", "Agent has to stop.");
      g_agentRequestId = -1;
      return err;
    }
    
    // Clean options
    mappingRemove(options, "agentRequestID");
    mappingRemove(options, "agentProjectID");
    mappingRemove(options, "agentAction");
    mappingRemove(options, "fwInstZipPath");
    
    PVSSBootstrapper_log("INFO", "Finished a request!");
    g_agentRequestId = -1; // Over
  }
}

int PVSSBootstrapper_executeOneShot (mapping &options) {
  int err;
  
  err = PVSSBootstrapper_doFunction(options);
  if(err < 0)
    return err;
    
  return 0;
}
