// Need option projectName
// Could need option projectPath
// Could set option projectName
// Need option user
// Need option password
// Need option openConsole
// Need option openLogViewer
// Need option onlyPmon
// Could need option dontRegister
// Set option pmonPort
int PVSSBootstrapper_startProject(mapping &options) { //TODO: Migrate parameters user and password to options
  int err, systemErr;
  
  // Get projectName
  string projName = options["projectName"];
  if(projName == "") {
    string projectPath = options["projectPath"];
    strreplace(projectPath, "\\", "/"); // For windows path
    dyn_string pathSplitted = strsplit(projectPath, "/");
    
    projName = (dynlen(pathSplitted)>0)?pathSplitted[dynlen(pathSplitted)]:"";
    options["projectName"] = projName;
  }
  
  // If project is not registered, register it
  if(!_PVSSBootstrapper_checkProjectExists(projName)) {
    // In case we don't want or can't register it
    if(options["dontRegister"] || (options["projectPath"] == "")) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_startProject() -> Unable to find the project " + projName + " in the list of registered projects");
      return -1;
    }
    
    options["pmonPort"] = 0; // Needed option of registerProject
    err = PVSSBootstrapper_registerProject(options);
    if(err < 0)
      return err;
     
    // Double-check registration
    if(!_PVSSBootstrapper_checkProjectExists(projName)) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_startProject() -> Unable to register the project");
      return -1;
    }
  }
  
  // If project is already running
  if(paIsPmonRunning(projName)) {
    if(options["restart"]) {
      err = PVSSBootstrapper_stopProject(options);
      if(err < 0)
        return -1;
    }else{
      PVSSBootstrapper_log("WARNING", "PVSSBootstrapper_startProject() -> Project " + projName + " already running");
      return 0;
    }
  }

  // Start project
  PVSSBootstrapper_log("INFO", "Starting project " + projName);
  
  err = pmonStartProject(systemErr, projName, makeDynAnytype(),
                         !options["onlyPmon"] /*autostart*/, options["openConsole"], options["user"], options["password"], options["openLogViewer"]);
  if(err < 0) {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_startProject() -> pmonStartProject returned error " + err + " and system error " + systemErr);
    return -1;
  }
  
  // Wait project is started
  if(!options["dontWait"] && !options["onlyPmon"]) {
    err = PVSSBootstrapper_waitProjectIsStarted(options);
    if(err < 0)
      return -1;
  }

  return 0;
}

// Need option projectName
int PVSSBootstrapper_stopProject(mapping &options) {
  int err, systemErr;
  
  string projName = options["projectName"];
  
  if(_PVSSBootstrapper_checkProjectExists(projName)) {
    if(!paIsPmonRunning(projName)) {
      PVSSBootstrapper_log("WARNING", "Project " + projName + " already stopped");
      return 0;
    }
  
    PVSSBootstrapper_log("INFO", "Stopping project " + projName);
      
    err = pmonStopProject(systemErr, projName);
    if(err < 0) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_stopProject() -> pmonStopProject returned error " + err + " and system error " + systemErr);
      return err;
    }
  }else{
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_stopProject() -> Project " + projName + " is not registered");
    return -1;
  }
  
  return 0;
}

int PVSSBootstrapper_stopAllProjects() {
  int err, systemErr;
  
  dyn_string projsList = makeDynString();
  dyn_string projsPath = makeDynString();
  
  err = _PVSSBootstrapper_checkPaGetProjs(projsList, projsPath);
  if(err < 0)
    return err;
  
  for(int i=1 ; i <= dynlen(projsList) ; i++) {
    if(paIsPmonRunning(projsList[i])) {
      PVSSBootstrapper_log("INFO", "Stopping project " + projsList[i]);
        
      err = pmonStopProject(systemErr, projsList[i]);
      if(err < 0) {
        PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_stopAllProjects() -> pmonStopProject returned error " + err + " and system error " + systemErr);
        return err;
      }
    }
  }
  
  return 0;
}

// Need option projectName
int PVSSBootstrapper_isProjectRunning(mapping &options) {
  int err;
  dyn_dyn_string ddsResult;
  
  string projName = options["projectName"];
  
  if(_PVSSBootstrapper_checkProjectExists(projName)) {
    if(!paIsPmonRunning(projName)) {
      PVSSBootstrapper_log("INFO", "Project " + projName + " is not running, pmon is off.");
      PVSSBootstrapper_setReturnData("False");
      return 0;
    }
    
    err = PVSSBootstrapper_pmonGetManStati(options, ddsResult);
    if(err < 0)
      return err;
    
    bool projStarted=true;
    for(int i=1 ; i < dynlen(ddsResult) ; i++) {
      if((ddsResult[i][3] == 2) && (ddsResult[i][1] != 2)) {
        projStarted = false;
        break;
      }
    }
    
    if(projStarted) {
      PVSSBootstrapper_log("INFO", "Project " + projName + " is running.");
      PVSSBootstrapper_setReturnData("True");
      return 0;
    }else{
      PVSSBootstrapper_log("INFO", "Project " + projName + " is not running, not all always managers are running correctly.");
      PVSSBootstrapper_setReturnData("False");
      return 0;
    }
    
  }else{
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_isProjectRunning() -> Project " + projName + " is not registered");
    return -1;
  }
}

// Need option projectPath
// Need option pmonPort
// Set option projectName
int PVSSBootstrapper_registerProject(mapping &options) {
  int err;
  
  // Determine project name
  string projectPath = options["projectPath"];
  strreplace(projectPath, "\\", "/"); // For windows path
  dyn_string pathSplitted = strsplit(projectPath, "/");
  projectPath = options["projectPath"]; // Put back the windows path for string comparison afterwards
  
  string projName = (dynlen(pathSplitted)>0)?pathSplitted[dynlen(pathSplitted)]:"";
  options["projectName"] = projName;
  
  if((strrtrim(strltrim(projectPath)) == "") || (projName == "" )) {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_registerProject() -> Project path or project name empty"); 
    return -1;
  }
  
  // Check if already registered
  dyn_string projsList = makeDynString();
  dyn_string projsPath = makeDynString();
  
  err = _PVSSBootstrapper_checkPaGetProjs(projsList, projsPath);
  if(err < 0)
    return err;
  
  int i = dynContains(projsList, projName);
  if(i > 0) {
    //// If the given project path is just a project name OR if the registered project path found corresponds to the one given
    //if((options["startProjectPath"] == projName) || (projsPath[i] == options["startProjectPath"])) {
    // If the registered project path found corresponds to the one given
    if(projsPath[i] == projectPath) {
      PVSSBootstrapper_log("WARNING", "Project " + projName + " already registered in PVSS. Skip registration");
      return 0;
    }else{
      PVSSBootstrapper_log("ERROR", "A project named " + projName + " is already registered in PVSS with a different path: " + projsPath[i] + " ; " + projectPath);
      return -1;
    }
  }
  
  // Recreate the path
  projectPath = "";
  for(int i = 1 ; i < dynlen(pathSplitted) ; i++)
    projectPath += pathSplitted[i] + "/";
    
  // Register the project
  PVSSBootstrapper_log("INFO", "Registering project " + projName + " in " + projectPath);
  
  err = paRegProj(projName, projectPath, "", options["pmonPort"], false, false); // Documentation of this function in PVSS_PATH/api/include/PVSSUtil/PVSSUtilAdmin.hxx:129
  if(err != 0) {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_registerProject() -> paRegProj() returned an error: " + err + ". Registration incomplete");
    return -1;
  }
  
  return 0;
}

// Need option projectName
// Could need option dontStop
int PVSSBootstrapper_unregisterProject(mapping &options) {
  int err;
  
  string projName = options["projectName"];
  
  if(_PVSSBootstrapper_checkProjectExists(projName)) {
    if(paIsPmonRunning(projName)) {
      if(options["dontStop"]) {
        PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_unregisterProject() -> Project " + projName + " is running");
        return -1;
      }
      
      err = PVSSBootstrapper_stopProject(options);
      if(err < 0)
        return err;
    }
  
    err = paDelProj(projName, false);
    if(err != 0) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_unregisterProject() -> paDelProj() returned an error: " + err + ". Unregistration incomplete");
      return err;
    }
  }else{
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_unregisterProject() -> Project " + projName + " unknown");
    return -1;
  }

  return 0;
}

// Need option projectName
// Could need option dontStop
int PVSSBootstrapper_deleteProject(mapping &options) {
  int err;
  
  string projName = options["projectName"];
  
  if(_PVSSBootstrapper_checkProjectExists(projName)) {
    if(paIsPmonRunning(projName)) {
      if(options["dontStop"]) {
        PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_deleteProject() -> Project " + projName + " is running");
        return -1;
      }
      
      err = PVSSBootstrapper_stopProject(options);
      if(err < 0)
        return err;
    }
  
    err = paDelProj(projName, true);
    if(err != 0) {
      PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_deleteProject() -> paDelProj() returned an error: " + err + ". Deletion incomplete");
      return err;
    }
  }else{
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_deleteProject() -> Project " + projName + " unknown");
    return -1;
  }

  return 0;
}

// Need option projectName
int PVSSBootstrapper_isProjectRegistered(mapping &options) {
  int err;
  
  string projName = options["projectName"];
  
  if(_PVSSBootstrapper_checkProjectExists(projName)) {
    PVSSBootstrapper_log("INFO", "PVSSBootstrapper_isProjectRegistered() -> Project " + projName + " is registered");
    PVSSBootstrapper_setReturnData("True");
    return 0;
  }else{
    PVSSBootstrapper_log("INFO", "PVSSBootstrapper_isProjectRegistered() -> Project " + projName + " is not registered");
    PVSSBootstrapper_setReturnData("False");
    return 0;
  }
}

// Need option projectName
// Need option installPath
// Need option systemNumber
// Can use option systemName
// Need option overwrite
// Need option pmonPort
// Need option dataPort
// Need option eventPort
// Need option distPort
int PVSSBootstrapper_createProject (mapping &options) {
  PVSSBootstrapper_log("INFO", "Creating project " + options["projectName"] + " in " + options["installPath"] + ". Overwrite: " + options["overwrite"]);
  
  if(!mappingHasKey(options, "systemName"))
    options["systemName"] = "dist_" + options["systemNumber"];
    
  int err = paCreateProjExtend(
    options["projectName"],
    options["installPath"],
    makeDynString("en_US.iso88591") /* language */,
    options["systemNumber"],
    options["systemName"],
    makeDynString() /* redu host */,
    makeDynString() /* sub projs */,
    true /* runnable */,
    true /* remote installable */,
    true /* distributed */,
    makeDynString() /* distPeers */,
    true /* registerProj */,
    options["overwrite"],
    options["pmonPort"],
    options["dataPort"],
    options["eventPort"],
    0 /* reduPort */,
    0 /* splitPort */,
    options["distPort"],
    makeDynAnytype() /*configEntries */);
                               
  if(err < 0) { // Error handling. Trying to gess the error for comprehensible error report...
    dyn_string ds, dsVer, dsPath;
    paGetProjs(ds, dsVer, dsPath);
      
    if(strrtrim(strltrim(options["projectName"])) == "") {
      PVSSBootstrapper_log("ERROR", "Invalid project name (empty).");
      return err;
    }else if(strrtrim(strltrim(options["installPath"])) == "") {
      PVSSBootstrapper_log("ERROR", "Invalid target directory (empty).");
      return err;
    }else if(!options["overwrite"] && (dynContains(ds, options["projectName"]) >= 1)) {
      PVSSBootstrapper_log("ERROR", "Project is already registered. Use the overwrite option.");
      return err;
    }else if(!options["overwrite"] && (access(options["installPath"]+"/"+options["projectName"], F_OK) == 0)) {
      PVSSBootstrapper_log("ERROR", "Project already exists. Use the overwrite option.");
      return err;
    }else{
      PVSSBootstrapper_log("ERROR", "paCreateProjExtend() returned unidentified error " + err + ".");
      return err;
    }
  }
  
  return 0;
}

// Need option projectName
int PVSSBootstrapper_waitProjectIsStarted(mapping &options) {
  int err;
  dyn_dyn_string ddsResult;
    
  PVSSBootstrapper_log("INFO", "Waiting for project " + options["projectName"] + " to start correctly");
  
  bool done=false;
  while(!done) { // TODO: Implement timeout
    err = PVSSBootstrapper_pmonGetManStati(options, ddsResult);
    if(err < 0)
      return err;
      
    if(dynlen(ddsResult) == 0) {
      PVSSBootstrapper_log("ERROR", "Pmon returned an empty list of manager. Are you sure the pmon of the project is running? Check for port collision.");
      return -1;
    }
    
    bool projStarted=true;
    for(int i=1 ; i < dynlen(ddsResult) ; i++) {
      if((ddsResult[i][3] == 2) && (ddsResult[i][1] != 2)) {
        projStarted = false;
        break;
      }
    }
    
    if(projStarted) {
      done = true;
      break; // Redondant
    }
    
    delay(0, 500);
  }
  
  PVSSBootstrapper_log("INFO", "Project " + options["projectName"] + " fully started");
  
  return 0;
}

int _PVSSBootstrapper_checkPaGetProjs(dyn_string &projsList, dyn_string &projsPath) {
  int err;
  
  dyn_string projsVersion = makeDynString();
  dyn_string badProjs = makeDynString();
  
  err = paGetProjs(projsList, projsVersion, projsPath, badProjs);
  if(err < 0) {
    if(dynlen(badProjs) > 0)
      PVSSBootstrapper_log("WARNING", "_PVSSBootstrapper_checkPaGetProjs() -> You have " + dynlen(badProjs) + " bad projec" + ((dynlen(badProjs) > 1) ? "ts" : "t") + " registered.");
    else{
      PVSSBootstrapper_log("ERROR", "_PVSSBootstrapper_checkPaGetProjs() -> paGetProjs returned error " + err);
      return err;
    }
  }
  
  return 0;
}

int _PVSSBootstrapper_checkProjectExists(string projName) {
  int err;
  
  dyn_string projsList = makeDynString();
  dyn_string projsPath = makeDynString();
  
  err = _PVSSBootstrapper_checkPaGetProjs(projsList, projsPath);
  if(err < 0)
    return err;
  
  return dynContains(projsList, projName);
}

// Need option projectName
// Set option projectPath
int PVSSBootstrapper_getProjectPath(mapping &options) {
  int err;
  
  err = _PVSSBootstrapper_resolveProjectPath(options);
  if(err < 0)
    return err;
    
  PVSSBootstrapper_log("INFO", "Project " + options["projectName"] + " has path " + options["projectPath"]);
  PVSSBootstrapper_setReturnData(options["projectPath"]);
  return 0;
}

int PVSSBootstrapper_listProjects() {
  int err;
  
  dyn_string projsList = makeDynString();
  dyn_string projsPath = makeDynString();
  
  err = _PVSSBootstrapper_checkPaGetProjs(projsList, projsPath);
  if(err < 0)
    return err;
  
  returnData = "";
  for(int i=1 ; i <= dynlen(projsList) ; i++) {
    PVSSBootstrapper_log("INFO", projsList[i]);
    returnData += projsList[i];
    if(i < dynlen(projsList))
      returnData += "/";
  }
  
  PVSSBootstrapper_setReturnData(returnData);
  return 0;
}

// Need option projectName
// Set option projectPath
int _PVSSBootstrapper_resolveProjectPath(mapping &options) {
  int err;
  string projName = options["projectName"];
  
  dyn_string projsList = makeDynString();
  dyn_string projsPath = makeDynString();
  
  err = _PVSSBootstrapper_checkPaGetProjs(projsList, projsPath);
  if(err < 0)
    return err;
  
  int i = dynContains(projsList, projName);
  if(i <= 0) {
    PVSSBootstrapper_log("ERROR", "_PVSSBootstrapper_resolveProjectPath() -> Project " + projName + " unknown");
    return -1;
  }
  
  string projectPath = projsPath[i];
  options["projectPath"] = projectPath;
  
  return 0;
}

// FwInstallation stuffs

// Need option fwInstZip
// Need option projectName
// Set option projectPath
int PVSSBootstrapper_deployZip(mapping &options) {
  int err;
  
  // Get the projectPath in the options mapping
  err = _PVSSBootstrapper_resolveProjectPath(options);
  if(err < 0)
    return err;
    
  string logFile = PROJ_PATH + "log/unzip.log";
  
  PVSSBootstrapper_log("INFO", "Unzip file from " + options["zipFile"] + " into " + options["projectPath"]);
  
  err = system("unzip -o \"" + options["zipFile"] + "\" -d \"" + options["projectPath"] + "/\" > " + logFile + " 2>&1");
  
  if(err) {
    PVSSBootstrapper_log("ERROR", "PVSSBootstrapper_deployZip() -> System call to the unzip tool returned an error " + err);
    return -1;
  }
  
  return 0;
}

// Need option projectName
// Set option projectPath
// Need option debug
int PVSSBootstrapper_startInsider(mapping &options) {
  int err;
  
  // Get the projectPath in the options mapping
  err = _PVSSBootstrapper_resolveProjectPath(options);
  if(err < 0)
    return err;
  
  // Place the bootstrapper in the projectPaths of the project
  err = _PVSSBootstrapper_prepareProjectPath(options);
  if(err < 0)
    return err;
  
  // Execute the insider control script in the project
  err = _PVSSBootstrapper_startCtrl(options, "libs/PVSSBootstrapper/PVSSBootstrapper_insider.ctl");
  if(err < 0)
    return err;
      
  err = _PVSSBootstrapper_cleanProjectPath(options);
  if(err < 0)
    return err;
      
  // In case the project restarted
  err = PVSSBootstrapper_waitProjectIsStarted(options);
  if(err < 0)
    return;
  
  return 0;
}

// Need option projectPath
int _PVSSBootstrapper_prepareProjectPath(mapping &options) {
  string projectPath = options["projectPath"];
  string configFile = projectPath + "/config/config";

  // Get previous list of project paths
  dyn_string values=makeDynString();
  paCfgReadValueList(configFile, "general", "proj_path", values);
  
  // Delete all // Actually, it does not delete all, but just the first occurence... wrong doc from PVSS !@?%&# Have to loop on it until it returns -1.
  while(paCfgDeleteValue(configFile, "general", "proj_path") == 0);
  
  // Insert the one from the bootstrapper path (priority order = inverse of declaration order)
  paCfgInsertValue(configFile, "general", "proj_path", dirName(getPath(SCRIPTS_REL_PATH)));
  // Re-insert all the other that were here before
  for(int i=1 ; i <= dynlen(values) ; i++)
    paCfgInsertValue(configFile, "general", "proj_path", values[i]);
  
  return 0;
}

// Need option projectPath
int _PVSSBootstrapper_cleanProjectPath(mapping &options) {
  string configFile = options["projectPath"] + "/config/config";
  paCfgDeleteValue(configFile, "general", "proj_path", dirName(getPath(SCRIPTS_REL_PATH)));
  
  return 0;
}

// Post processing stuffs

// Need option postProcessScript
int PVSSBootstrapper_postProcess(mapping &options) {
  int err;
  dyn_string scripts = options["postProcessScript"];
  
  for(int i=1 ; i <= dynlen(scripts) ; i++) {
    int sepPos = strpos(scripts[i], ":");
    string type = strtoupper(substr(scripts[i], 0, sepPos));
    string script = substr(scripts[i], sepPos+1);
    switch(type) {
      case "PANEL":
        err = _PVSSBootstrapper_startPanel(options, script);
      break;
      case "CTRL":
        err = _PVSSBootstrapper_startCtrl(options, script);
      break;
      case "CMD":
        err = _PVSSBootstrapper_startCommand(options, script);
      break;
      default:
        PVSSBootstrapper_log("ERROR", "Unknown type of script: " + type);
        return -1;
    }
    if(err < 0)
      return err;
  }
  
  return 0;
}

// Need option projectName
int _PVSSBootstrapper_startPanel(mapping &options, string panel) {
  string man = _PVSSBootstrapper_getWCCOAExecutable("ui");
  if(_WIN32)
    man += ".exe";
    
  PVSSBootstrapper_log("INFO", "Executing PVSS panel " + panel);
    
  strreplace(panel, "$", "\\\\$");
  system(man + " -proj " + options["projectName"] + " -p " + panel + " -iconBar -menuBar");
  
  return 0;
}

// Need option projectName
// Need option projectPath
int _PVSSBootstrapper_startCtrl(mapping &options, string script) {
  string man = _PVSSBootstrapper_getWCCOAExecutable("ctrl");
  if(_WIN32)
    man += ".exe";
  
  PVSSBootstrapper_log("INFO", "Executing PVSS CTRL script " + script);
    
  system(man + " -proj " + options["projectName"] + " " + script + ((options["debug"]) ? " -log +stderr" : ""));
  
  return 0;
}

int _PVSSBootstrapper_startCommand(mapping &options, string cmd) {  
  PVSSBootstrapper_log("INFO", "Executing system command " + cmd);
  
  system(cmd);
  
  return 0;
}

// Raw and shameless copy of fwInstallation_getWCCOAExecutable
string _PVSSBootstrapper_getWCCOAExecutable(string type) {
  string prefix = "PVSS00";

  if(VERSION_DISP != "3.8-SP2" && VERSION_DISP != "3.8") {
    if(strtolower(type) == "data" ||
       strtolower(type) == "event" ||
       strtolower(type) == "dist" ||
       strtolower(type) == "sim" ||
       strtolower(type) == "redu" ||
       strtolower(type) == "split" ||
       strtolower(type) == "proxy" ||
       strtolower(type) == "databg" ||
       strtolower(type) == "pmon" 
       )
      prefix = "WCCIL";    
    else
      prefix = "WCCOA";
  }
  
  return prefix + strtolower(type);
}
