#!/usr/bin/python
import os
import sys
import platform
import shutil
import urlparse
import tempfile
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapper import WinCCOA
from PVSSBootstrapper.WinCCOAProject import WinCCOAProject

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

def haveClassFixture(clsToDecorate):
    """Class decorator. Needed to keep class fixture functionality of unittest working with the layer concept of zope.testrunner.
       Add a Layer (in the zope.testrunner sense) capable of calling the setUp and tearDown class methods of the TestCase class to decorate.
       Use like this:
       
       @haveClassFixture
       class MyTestCase(unnittest.TestCase):
           ...
       """
    class ClassFixtureLayer:
        @classmethod
        def setUp(cls):
            clsToDecorate.setUpClass()
            
        @classmethod
        def tearDown(cls):
            clsToDecorate.tearDownClass()
            
    clsToDecorate.layer = ClassFixtureLayer
    clsToDecorate.layer.__name__ = clsToDecorate.__name__ + '.' + clsToDecorate.layer.__name__
    return clsToDecorate

class PBSTestCase(unittest.TestCase):
    
    def __init__(self, arg):
        super(PBSTestCase, self).__init__(arg)
        haveClassFixture(self.__class__) # Use it directly rather than as a class decorator. It makes the job effortless for all the subclasses.

    @classmethod
    def setUpClass(cls):
        cls.sandbox = 'Sandbox' # Current working directory should be tests/. Please, keep it relative to ensure tests using relative paths work.
        cls.sandbox = tempfile.mkdtemp(prefix='WCCOABootstrapperTest-')
        if not os.path.exists(cls.sandbox):
            os.makedirs(cls.sandbox)
            
        cls.supportFiles = os.path.abspath('supportFiles')
        if not os.path.exists(cls.supportFiles):
            os.makedirs(cls.supportFiles)
            
        cls.wccoa = WinCCOA.getDefaultInstance()
            
    @classmethod
    def tearDownClass(cls):
        if os.path.exists(cls.sandbox):
            shutil.rmtree(cls.sandbox)
    
    @classmethod
    def execCTRLScript(cls, projectPath, projectName, scriptFile, debug=False):
        shutil.copy2(os.path.abspath(scriptFile), os.path.join(projectPath, 'scripts'))
        
        args = []
        if debug:
            args.append('log')
            args.append('+stderr')
        args.append(scriptFile)
        
        cls.wccoa.runManagerInProject('ctrl', projectName, *args)
        
    @staticmethod
    def setGediToManual(projectPath):
        proj = WinCCOAProject(projectPath)
        proj.updateConsoleManagers(*[p._replace(start='manual') for p in proj.filterProgs(manager='WCCOAui', options='-m gedi')])
        
    @staticmethod
    def setGediToOnce(projectPath):
        proj = WinCCOAProject(projectPath)
        proj.updateConsoleManagers(*[p._replace(start='once') for p in proj.filterProgs(manager='WCCOAui', options='-m gedi')])
        
    @classmethod
    def _urlgrabSupportFile(cls, url):
        destFolder = cls.supportFiles
        destFile = os.path.join(destFolder, os.path.basename(urlparse.urlsplit(url).path))
        
        if os.path.exists(destFile):
            return destFile
        else:
            return PVSSBootstrapper.grabURLFile(url, destFolder)
        
    @classmethod
    def getLatestFwInstallation(cls):
        return PBSTestCase._urlgrabSupportFile('https://edms.cern.ch/file/1085149/52/fwInstallation-6.2.4.zip')
        
    @classmethod
    def getLatestJCOP(cls):
        return PBSTestCase._urlgrabSupportFile('https://edms.cern.ch/file/1055675/5.0.0/framework5.0.0.zip')
        
    @classmethod
    def getLatestUNICOS(cls):
        return PBSTestCase._urlgrabSupportFile('https://edms.cern.ch/file/1176169/5.3.2.1/unicos-framework-5.3.2.zip')
  
def suite():
  return unittest.TestLoader().discover('.')
  
if __name__ == '__main__':
  unittest.TextTestRunner(verbosity=2).run(suite())
