#!/usr/bin/python

import re
import psutil
import itertools
import platform

from PVSSBootstrapper._python_compat import *

def allPVSSProcesses():
  return itertools.ifilter(lambda p: p.name.startswith('PVSS') or p.name.startswith('WCC'), psutil.process_iter())
      
def _matchAllArgs(iterable, filters):
  return itertools.ifilter(lambda p: all(map(lambda arg: arg in ' '.join(p.cmdline), filters)), iterable)
     
def projectManagers(projectName, manFilter=None):
  allManagersIter = _matchAllArgs(allPVSSProcesses(), ['-PROJ ' + projectName])
  
  # Removed the "return blabla() if bla else bla" for Pyton 2.4
  if manFilter:
    return itertools.ifilter(lambda p: re.search(manFilter, p.name) != None, allManagersIter)
  else:
    return allManagersIter
     
def getPmon(projectName):
    if platform.system() == 'Windows':
        return next(itertools.ifilter(
            lambda p: re.search('^(PVSS|WCCIL)pmon(\.exe)?$', p.name) != None, _matchAllArgs(
                allPVSSProcesses(), [projectName]
            )
        ), None)
    else:
        return next(projectManagers(projectName, '^(PVSS00|WCCIL)pmon(\.exe)?$'), None)

def getGedi(projectName):
  return next(_matchAllArgs(projectManagers(projectName, '^(PVSS00|WCCOA)ui(\.exe)?$'), ['-m gedi']), None)

def getConsole():
  return next(itertools.ifilter(
    lambda p: re.search('^(PVSS00|WCCOA)ui(\.exe)?$', p.name) != None, _matchAllArgs(
      allPVSSProcesses(), ['-console']
    )
  ), None)

def getLogViewer(projectName):
  return next(itertools.ifilter(
    lambda p: re.search('^(PVSS|WCCOA)toolLogViewer(\.exe)?$', p.name) != None, _matchAllArgs(
      allPVSSProcesses(), [projectName]
    )
  ), None)
      
      
# for m in PVSSManagers.getManagers(projectName, 'PVSS00valarch'): m.send_signal(9)

# pmon = PVSSManagers.getPmon(projectName)
# for f in pmon.get_open_files(): print f
# pmon.get_threads()
# for c in pmon.get_connections(): print c
