import os
import shutil
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_deleteProject"

class DeleteProjectTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.createProject(cls.sandbox, projectName, overwrite=True)
    cls.projectPath = os.path.join(cls.sandbox, projectName)
    cls.setGediToManual(cls.projectPath)
    
  @classmethod
  def tearDownClass(cls):
    # In case the project did get succesfully deleted
    if PVSSBootstrapper.isProjectRegistered(projectName):
      PVSSBootstrapper.deleteProject(projectName)
    else:
      if os.path.exists(cls.projectPath):
        shutil.rmtree(cls.projectPath)
        
    PBSTestCase.tearDownClass()

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.stopAllProjects()
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopAllProjects()
    
  def test_projectName_valid(self):
    PVSSBootstrapper.deleteProject(projectName)
    
    self.assertFalse(PVSSBootstrapper.isProjectRegistered(projectName))
    
    self.assertFalse(os.path.exists(self.projectPath), 'Deleted project files are still here!')
    
  def test_projectName_invalid(self):
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_deleteProject\(\) -> Project blablabla unknown') as cm:
      PVSSBootstrapper.deleteProject('blablabla')
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Deleting the project blablabla should have failed')
    
  def test_dontStop(self):
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_deleteProject\(\) -> Project ' + projectName + ' is running') as cm:
      PVSSBootstrapper.deleteProject(projectName, dontStop=True)
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Deleting the running project should have failed')
    
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName))
    self.assertTrue(PVSSBootstrapper.isProjectRegistered(projectName))
    
    self.assertTrue(os.path.exists(self.projectPath), 'Deleting the running project deleted the files!')
