import os
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_getProjectPath"

class GetProjectPathTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.createProject(cls.sandbox, projectName, overwrite=True)
    
    cls.projectPath = os.path.join(cls.sandbox, projectName)

  @classmethod
  def tearDownClass(cls):
    PVSSBootstrapper.deleteProject(projectName)
    PBSTestCase.tearDownClass()
    
  def test_getProjectPath(self):
    self.assertEqual(PVSSBootstrapper.getProjectPath(projectName), os.path.abspath(self.projectPath))
    
  def test_invalid(self):
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, '_PVSSBootstrapper_resolveProjectPath\(\) -> Project blablabla unknown') as cm:
      PVSSBootstrapper.getProjectPath('blablabla')
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Starting the unregistered project should have failed')
