import os
import zipfile
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

class GrabURLFileTest(PBSTestCase):
    
    @classmethod
    def setUpClass(cls):
        PBSTestCase.setUpClass()
        cls.destFolder = cls.sandbox
    
    def setUp(self):
        PBSTestCase.setUp(self)
        self.destFolderContent = os.listdir(self.destFolder)
        
    def tearDown(self):
        PBSTestCase.tearDown(self)
        
        for f in os.listdir(self.destFolder):
            if f not in self.destFolderContent:
                os.remove(os.path.join(self.destFolder, f))
                
    def test_EDMSFile(self):
        url = 'https://edms.cern.ch/file/1085149/52/fwInstallation-6.2.4.zip'
        urlFile = PVSSBootstrapper.grabURLFile(url, self.destFolder)
        
        self.assertEqual(os.path.join(self.sandbox, 'fwInstallation-6.2.4.zip'), urlFile, 'The returned file path is not the expected one')
        self.assertTrue(os.path.exists(urlFile), 'The downloaded file does not exists')
        self.assertEqual(os.path.getsize(urlFile), 746099, 'The downloaded file does not have the expected file size')
        self.assertTrue(zipfile.is_zipfile(urlFile), 'The downloaded file is not a valid zip file')
