import os
import shutil
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_installComponent"

class InstallComponentTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    
    cls.fwInstallation = PBSTestCase.getLatestFwInstallation()
    cls.JCOP = PBSTestCase.getLatestJCOP()
    #cls.UNICOS = PBSTestCase.getLatestUNICOS()

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.createProject(self.sandbox, projectName, overwrite=True)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    PVSSBootstrapper.deployZip(projectName, self.fwInstallation)
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    PVSSBootstrapper.setComponentInstallationFolder(projectName, 'Components')
    
    self.projectPath = os.path.join(self.sandbox, projectName)
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopProject(projectName, warningLevel=0)
    PVSSBootstrapper.deleteProject(projectName)
    
  def test_installJCOP(self):
    PVSSBootstrapper.installComponent(projectName, sources=self.JCOP, components=['fw3DViewer.xml', 'fwAccessControl.xml', 'fwAnalogDigital.xml', 'fwCaen.xml', 'fwCaV.xml', 'fwConfigurationDB.xml', 'fwCore.xml', 'fwDIM.xml', 'fwDIP.xml', 'fwExternalApps.xml', 'fwFSMConfDB.xml', 'fwMCS.xml', 'fwTrending.xml', 'fwWiener.xml', 'fwXML.xml'])
    
    filesToCheck = ['fw3DViewer.xml', 'fwAccessControl.xml', 'fwAlarmHandling.xml', 'fwAnalogDigital.xml', 'fwCaen.xml', 'fwCaV.xml', 'fwConfigs.xml', 'fwConfigurationDB.xml', 'fwCore.xml', 'fwDevice.xml', 'fwDeviceEditorNavigator.xml', 'fwDIM.xml', 'fwDIP.xml', 'fwExternalApps.xml', 'fwFSM.xml', 'fwFSMConfDB.xml', 'fwGeneral.xml', 'fwMCS.xml', 'fwNode.xml', 'fwTrending.xml', 'fwWiener.xml', 'fwXML.xml', 'unDistributedControl.xml']
    for f in filesToCheck:
      self.assertTrue(os.path.exists(os.path.join(self.projectPath, 'Components', f)))
