import os
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_isProjectRegistered"

class IsProjectRegisteredTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.createProject(cls.sandbox, projectName, overwrite=True)
    cls.projectPath = os.path.join(cls.sandbox, projectName)
    cls.setGediToManual(cls.projectPath)

  @classmethod
  def tearDownClass(cls):
    PVSSBootstrapper.deleteProject(projectName)
    PBSTestCase.tearDownClass()

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.registerProject(self.projectPath, warningLevel=0)
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopAllProjects()
    
  def test_whenRegistered(self):
    self.assertTrue(PVSSBootstrapper.isProjectRegistered(projectName))
    
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName),  'The registered project should have started')
    
  def test_whenNotRegistered(self):
    PVSSBootstrapper.unregisterProject(projectName)
    self.assertFalse(PVSSBootstrapper.isProjectRegistered(projectName))
    
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_startProject\(\) -> Unable to find the project ' + projectName + ' in the list of registered projects') as cm:
      PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Starting the unregistered project should have failed')
