import os
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase
import PVSSManagers

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_isProjectRunning"

class IsProjectRunningTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.createProject(cls.sandbox, projectName, overwrite=True)
    
    cls.projectPath = os.path.join(cls.sandbox, projectName)

  @classmethod
  def tearDownClass(cls):
    PVSSBootstrapper.deleteProject(projectName)
    PBSTestCase.tearDownClass()

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopAllProjects()
    
  def test_whenRunning(self):
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName))
    
    # First check: pmon and gedi are running
    pmon = PVSSManagers.getPmon(projectName)
    self.assertTrue(pmon != None, 'The pmon of the project is not running')
    self.assertTrue(pmon.is_running(), 'The pmon of the project is not running')
    gedi = PVSSManagers.getGedi(projectName)
    self.assertTrue(gedi != None, 'The gedi of the project is not running')
    self.assertTrue(gedi.is_running(), 'The gedi of the project is not running')
  
    # Second check: try to execute a .ctl script
    self.execCTRLScript(self.projectPath, projectName, 'getSystemInfo.ctl')
    
    configFile = os.path.join(self.projectPath, 'config', 'config')
    kVs = PVSSBootstrapper._readConfigFile(configFile)
    
    self.assertTrue('test_systemNumber' in kVs, 'Could not detect the system number of the project')
    self.assertEqual(int(kVs['test_systemNumber']), 1, 'The system number of the project is not the expected one')
    
    self.assertTrue('test_systemName' in kVs, 'Could not detect the system name of the project')
    self.assertEqual(kVs['test_systemName'], 'dist_1', 'The system name of the project is not the expected one')
    
  def test_whenNotRunning(self):
    PVSSBootstrapper.stopProject(projectName)
    self.assertFalse(PVSSBootstrapper.isProjectRunning(projectName))
    
    # First check: pmon and gedi are not running
    pmon = PVSSManagers.getPmon(projectName)
    self.assertFalse(pmon != None, 'The pmon of the project is running')
    gedi = PVSSManagers.getGedi(projectName)
    self.assertFalse(gedi != None, 'The gedi of the project is running')
