import os
from PVSSBootstrapper import PVSSBootstrapper, WinCCOA
from PVSSBootstrapperTests import PBSTestCase

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_listProjects"

class ListProjectsTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.createProject(cls.sandbox, projectName, overwrite=True)
    PVSSBootstrapper.createProject(cls.sandbox, projectName + '2', overwrite=True)

  @classmethod
  def tearDownClass(cls):
    PVSSBootstrapper.deleteProject(projectName)
    PVSSBootstrapper.deleteProject(projectName + '2')
    PBSTestCase.tearDownClass()
    
  def test_listProjects(self):
    projsList = PVSSBootstrapper.listProjects()
    
    self.assertIn(projectName, projsList, 'Project 1 is not listed')
    self.assertIn(projectName + '2', projsList, 'Project 2 is not listed')
    
  def test_noProject(self):
    # Make up a fake WinCC OA registry
    pvssIIroot = os.path.join(self.sandbox, 'config')
    if not os.path.exists(pvssIIroot):
      os.mkdir(pvssIIroot)
      
    with open(os.path.join(pvssIIroot, 'pvssInst.conf'), 'w') as pvssInst:
      pvssInst.write('[Software\\ETM\\PVSS II\\%s]\n' % self.wccoa.version)
      pvssInst.write('InstallationDir = "%s"\n' % self.wccoa.path)
  
    backupEnv = os.environ.copy()
    os.environ['PVSS_II_ROOT'] = pvssIIroot
  
    projsList = PVSSBootstrapper.listProjects()
    
    os.environ = backupEnv
    
    self.assertEqual(len(projsList), 0, 'The list of project is not empty')
