import os
import shutil
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase
import PVSSManagers

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_setComponentInstallationFolder"

class SetComponentInstallationFolderTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    
    cls.fwInstallation = PBSTestCase.getLatestFwInstallation()
    cls.JCOP = PBSTestCase.getLatestJCOP()

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.createProject(self.sandbox, projectName, overwrite=True)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    PVSSBootstrapper.deployZip(projectName, self.fwInstallation)
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    
    self.projectPath = os.path.join(self.sandbox, projectName)
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopProject(projectName, warningLevel=0)
    
    PVSSBootstrapper.deleteProject(projectName)
    
  def test_absolute(self):
    try:
        compPath = os.path.abspath(os.path.join(self.sandbox, projectName + '_components'))
        PVSSBootstrapper.setComponentInstallationFolder(projectName, compPath)
        
        self.assertTrue(os.path.exists(compPath), 'Component folder do not exist')
        
        # Test it works by installing fwCore and check at least the XML file is in the folder
        PVSSBootstrapper.installComponent(projectName, sources=self.JCOP, components='fwCore.xml')
        self.assertTrue(os.path.exists(os.path.join(compPath, 'fwCore.xml')))
        
    finally:
        if os.path.exists(compPath):
            PVSSBootstrapper.stopProject(projectName)
            shutil.rmtree(compPath)
    
  def test_relative(self):
    try:
        compPath = os.path.join(self.projectPath, 'Components')
        PVSSBootstrapper.setComponentInstallationFolder(projectName, 'Components')

        self.assertTrue(os.path.exists(compPath), 'Component folder do not exist')
        
        # Test it works by installing fwCore and check at least the XML file is in the folder
        PVSSBootstrapper.installComponent(projectName, sources=self.JCOP, components='fwCore.xml')
        self.assertTrue(os.path.exists(os.path.join(compPath, 'fwCore.xml')))
    finally:
        if os.path.exists(compPath):
            PVSSBootstrapper.stopProject(projectName)
            shutil.rmtree(compPath)
