import os
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase
import PVSSManagers

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_stopAllProjects"

class StopAllProjectsTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.createProject(cls.sandbox, projectName, pmonPort=5000, dataPort=5001, eventPort=5002, distPort=5003, overwrite=True)
    cls.setGediToManual(os.path.join(cls.sandbox, projectName))
    PVSSBootstrapper.createProject(cls.sandbox, projectName + '2', pmonPort=5010, dataPort=5011, eventPort=5012, distPort=5013, overwrite=True)
    cls.setGediToManual(os.path.join(cls.sandbox, projectName + '2'))

  @classmethod
  def tearDownClass(cls):
    PVSSBootstrapper.deleteProject(projectName)
    PVSSBootstrapper.deleteProject(projectName + '2')
    PBSTestCase.tearDownClass()

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    PVSSBootstrapper.startProject(projectName + '2', noConsole=True, noLogViewer=True)
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopProject(projectName, warningLevel=0)
    PVSSBootstrapper.stopProject(projectName + '2', warningLevel=0)
    
  def test_stopAllProjects(self):
    PVSSBootstrapper.stopAllProjects()
    self.assertFalse(PVSSBootstrapper.isProjectRunning(projectName), 'Project 1 is still running')
    self.assertFalse(PVSSBootstrapper.isProjectRunning(projectName + '2'), 'Project 2 is still running')
    
    pmon1 = PVSSManagers.getPmon(projectName)
    self.assertTrue(pmon1 == None, 'The pmon of project 1 is still running')
    pmon2 = PVSSManagers.getPmon(projectName + '2')
    self.assertTrue(pmon2 == None, 'The pmon of project 2 is still running')
