import os
import PVSSBootstrapper
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase
import PVSSManagers

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_stopProject"

class StopProjectTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.createProject(cls.sandbox, projectName, overwrite=True)
    cls.setGediToManual(os.path.join(cls.sandbox, projectName))

  @classmethod
  def tearDownClass(cls):
    PVSSBootstrapper.deleteProject(projectName)
    PBSTestCase.tearDownClass()

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopAllProjects()
    
  def test_projectName_valid(self):
    PVSSBootstrapper.stopProject(projectName)
    self.assertFalse(PVSSBootstrapper.isProjectRunning(projectName), 'The project is still running')
    
    pmon = PVSSManagers.getPmon(projectName)
    self.assertTrue(pmon == None, 'The pmon of the project is still running')
    
  def test_projectName_invalid(self):
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_stopProject\(\) -> Project blablabla is not registered') as cm:
      PVSSBootstrapper.stopProject('blablabla')
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Stopping project blablabla should have failed')
