import os
import shutil
import unittest
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_unregisterProject"

class UnregisterProjectTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.createProject(cls.sandbox, projectName, overwrite=True)
    cls.projectPath = os.path.join(cls.sandbox, projectName)
    cls.setGediToManual(cls.projectPath)

  @classmethod
  def tearDownClass(cls):
    # In case the project did get succesfully registered
    if PVSSBootstrapper.isProjectRegistered(projectName):
      PVSSBootstrapper.deleteProject(projectName)
    else:
       if os.path.exists(cls.projectPath):
         shutil.rmtree(cls.projectPath)
        
    PBSTestCase.tearDownClass()

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.registerProject(self.projectPath, warningLevel=0)
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopAllProjects()
    
  def test_projectName_valid(self):
    PVSSBootstrapper.unregisterProject(projectName)
    
    self.assertFalse(PVSSBootstrapper.isProjectRegistered(projectName))
    
    self.assertTrue(os.path.exists(self.projectPath), 'Unregistering the project deleted the files!')
    
  def test_projectName_invalid(self):
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_unregisterProject\(\) -> Project blablabla unknown') as cm:
      PVSSBootstrapper.unregisterProject('blablabla')
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Unregistering the project blablabla should have failed')
    
  def test_dontStop(self):
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_unregisterProject\(\) -> Project ' + projectName + ' is running') as cm:
      PVSSBootstrapper.unregisterProject(projectName, dontStop=True)
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Unregistering the running project should have failed')
    
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName))
    self.assertTrue(PVSSBootstrapper.isProjectRegistered(projectName))
    
    self.assertTrue(os.path.exists(self.projectPath), 'Unregistering the project deleted the files!')
