#uses "fwBenchmark/fwBenchmark_Util.ctl"
#uses "fwBenchmark/fwBenchmark_DpSetAttributes.ctl"
#uses "fwBenchmark/fwBenchmark_DpGetAttributes.ctl"
#uses "fwBenchmark/fwBenchmark_DpCreate.ctl"
#uses "fwBenchmark/fwBenchmark_Const.ctl"
/*
 * 
 * This script executes the standard benchmarks to 
 * evaluate the performances of a machine or of a 
 * new WinCC OA version/patch
 * 
 * jean-charles.tournier@cern.ch
 * CERN - EN/ICE/SCD
 * 
 * July 14th, 2015
 * 
 */
 
 //Location of the script results
 const string FWBM_RESULTS_FOLDER 			= "fwBenchmark/"; //getPath(DATA_REL_PATH) is added to the folder name
 const string FWBM_RESULTS_FILE_EXTENSION 	= ".txt";
 
 //Constant for the parameters of the benchmark
 //they should not be modified as it is hard afterwards to compare previous results
 const int FWBM_NUM_ATTRIBUTES  	= 10000; 	//number of attributes, dpSet, dpGet, etc.
 const int FWBM_BUNCH_STEP_GET 		= 100;		//step increment used by the dpGetValue  benchmark 		
 const int FWBM_BUNCH_STEP_SET 		= 100;		//step increment used by the dpSetScalar benchmark			
 
 
 //delay between the execution of two benchmark in second
 const unsigned FWBM_INTERBENCHMARK_DELAY = 1;
 

/** 
 *  Script entry point which launches
 *  all the different benchmarks
 *  
 */
 void main(...)
 {
 	
 	string sFileName = "";
 	dyn_float dfNotUsed = makeDynFloat(0);
 	dyn_dyn_anytype ddaNotUsed;
 	dyn_string dsListRecipients;
 	
 	//Reports and email destination names
 	string sDateTimePrefixReports = fwBenchmark_Util_getDateTimePrefix() + "fwBenchmarkResults";
 	string sNameResultsFolder = getPath(DATA_REL_PATH)+FWBM_RESULTS_FOLDER+sDateTimePrefixReports+"/";
 	string sNameResultsZip = sDateTimePrefixReports;
 	
 	//make directory and make sure the folder holding the report exists
 	if (mkdir(sNameResultsFolder) == FALSE)
 	{
 		fwBM_log( FWBM_ERROR_LOG, __FILE__, "The folder containing the benchmark result can not be created ("+sNameResultsFolder+")");
 		fwBM_log( FWBM_ERROR_LOG, __FILE__, "Stopping the benchmark due to error.");
 		return;
 	} 
 	
 	//--------------------------------------------
 	// 0 - Check if reports need to be sent
 	//--------------------------------------------
 	va_list args;
 	int len = va_start(args);  // Returns the number of parameters  
    for (int i=1; i<=len; i++) 
	{    
    	if (patternMatch("-mail",args[i]))
		{ 
			dynAppend(dsListRecipients, args[i+1]);
		} 
    }
    va_end(args); // va_end stops the handling of the list
 	if (dynlen(dsListRecipients) < 1)
 	{
 		fwBM_log( FWBM_INFO_LOG, "", "************************************************************************");
 		fwBM_log( FWBM_INFO_LOG, "", "The reports will not be sent by email");
 		fwBM_log( FWBM_INFO_LOG, "", "You can specify the argument -mail to have the report sent by email:");
 		fwBM_log( FWBM_INFO_LOG, "", "e.g. -mail tournier@cern.ch");
 		fwBM_log( FWBM_INFO_LOG, "", "Stop the CTRL manager now if you want to add the sending option.");
 		fwBM_log( FWBM_INFO_LOG, "", "************************************************************************");
 		for( int i=10; i>=0; i--) //Give 10 seconds to the user to stop the manager if needed
 		{
 			fwBM_log( FWBM_INFO_LOG, "", "Starting benchmarks in "+i+" seconds");
 			delay(1);
 		}
 	} else
 	{
 		fwBM_log( FWBM_INFO_LOG, __FILE__, "Reports will be sent to "+(string)dsListRecipients);
 	}
 	
 	//--------------------------------------------
 	// 1 - Launch the dpCreate Benchmark
 	//--------------------------------------------
 	fwBenchmark_dpCreate_Clean();
 	delay(FWBM_INTERBENCHMARK_DELAY);
 	sFileName = sNameResultsFolder+fwBenchmark_Util_getDateTimePrefix() + "dpCreate"+FWBM_RESULTS_FILE_EXTENSION;
 	if (fwBenchmark_dpCreate_Launch( FWBM_NUM_ATTRIBUTES, sFileName, dfNotUsed) == FWBM_PVSS_ERROR)
 	{
 		fwBM_log( FWBM_ERROR_LOG, __FILE__, "Stopping the benchmark due to error.");
 		return;
 	}
 	//fwBenchmark_Util_sendEmail( dsListRecipients, "fwBenchmark - dpCreate", "", sFileName);
 	delay(FWBM_INTERBENCHMARK_DELAY);

 	//--------------------------------------------		
 	//	2 - Main loop dpSetWait and dpGet for all the configurations.
 	//		You will have to write the different configurations that are to be performed, and the sequence.
 	//		The possibilities are:
 	//		FWBM_CONFIG_ORIGINALVALUE, for Original Value,
 	//		FWBM_CONFIG_DRIVER_SIM, for Simulator Driver,
 	//		FWBM_CONFIG_DRIVER_S7, for Simatic S7 Driver,
 	//		FWBM_CONFIG_DRIVER_MODBUS for Modbus Driver,
 	//		FWBM_CONFIG_DRIVER_IEC, for IEC Driver,
 	//		FWBM_CONFIG_ARCHIVE for Archive,
 	//		FWBM_CONFIG_ALERTHDL for Alert Handling, but this configuration can only occur under certain conditions (NUM_ATTRIBUTES and STEP multiples of 14)
 	//-------------------------------------------- 	
 	
 	dyn_string dsConfiguration;
 	dynAppend(dsConfiguration,FWBM_CONFIG_ORIGINALVALUE);
 	dynAppend(dsConfiguration,FWBM_CONFIG_DRIVER_SIM);
	dynAppend(dsConfiguration,FWBM_CONFIG_DRIVER_S7);
 	dynAppend(dsConfiguration,FWBM_CONFIG_DRIVER_MODBUS);
 	dynAppend(dsConfiguration,FWBM_CONFIG_DRIVER_IEC);
 	dynAppend(dsConfiguration,FWBM_CONFIG_ARCHIVE);
 	dynAppend(dsConfiguration,FWBM_CONFIG_ALERTHDL);

	//Clean the previously DPE (otherwise, error to overwrite with the address)
	fwBenchmark_dpCreate_Clean();
	
 	float fTotalElapsedTime = 0;
 	//Launch all configurations
 	const int iConfigurationLength = dynlen(dsConfiguration); 
 	for(int i = 1; i<=iConfigurationLength;i++)
 	{
 		time tStart, tEnd;
 		tStart = getCurrentTime();
 	
 		fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "***Starting '"+dsConfiguration[i]+"' configuration***");
 		
 		//**Set configuration
 		string sBaseFileNameSet = "dpSet" + dsConfiguration[i];
 		sFileName = sNameResultsFolder+fwBenchmark_Util_getDateTimePrefix() +
 			sBaseFileNameSet+FWBM_RESULTS_FILE_EXTENSION;
 			
 		if (fwBenchmark_dpSetAttributes_Launch( FWBM_NUM_ATTRIBUTES, FWBM_BUNCH_STEP_SET, dsConfiguration[i], sFileName, ddaNotUsed) == FWBM_PVSS_ERROR)
	 	{
	 		fwBM_log( FWBM_ERROR_LOG, __FILE__, "Stopping the benchmark due to error.");
	 		return;
	 	}
	 	
	 	//fwBenchmark_Util_sendEmail( dsListRecipients, "fwBenchmark - "+sBaseFileNameSet, "", sFileName);
	 	delay(FWBM_INTERBENCHMARK_DELAY);
	 	
	 	//**Get configuration
	 	string sBaseFileNameGet = "dpGet" + dsConfiguration[i];
 		sFileName = sNameResultsFolder+fwBenchmark_Util_getDateTimePrefix() +
 			sBaseFileNameGet+FWBM_RESULTS_FILE_EXTENSION;
 			
 		if (fwBenchmark_dpGetAttributes_Launch( FWBM_NUM_ATTRIBUTES, FWBM_BUNCH_STEP_GET, dsConfiguration[i], sFileName, ddaNotUsed) == FWBM_PVSS_ERROR)
	 	{
	 		fwBM_log( FWBM_ERROR_LOG, __FILE__, "Stopping the benchmark due to error.");
	 		return;
	 	}
	 	
	 	//fwBenchmark_Util_sendEmail( dsListRecipients, "fwBenchmark - "+sBaseFileNameGet, "", sFileName);
	 	delay(FWBM_INTERBENCHMARK_DELAY);
	 	
	 	//get the total time to complete the test
		tEnd = getCurrentTime();
		float fElapsedTime =  fwBenchmark_Util_computeElapsedTime( tStart, tEnd);
 		fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "******Time in Set and Get of " + dsConfiguration[i]
 			+ " : " + fElapsedTime
 		);
 		fTotalElapsedTime = fTotalElapsedTime + fElapsedTime;

 	}
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "******Total time in Set and Get: " + fTotalElapsedTime);
 	
 	//Create a zip folder with all the results and send it by e-mail.
 	if (_UNIX)
	{
		if(system("cd "+sNameResultsFolder+" && zip "+sNameResultsZip + " -name *.txt") == 0)
	 	{
	 		delay(FWBM_INTERBENCHMARK_DELAY);
	 		fwBenchmark_Util_sendEmail( dsListRecipients, "fwBenchmark - "+sDateTimePrefixReports, "", sNameResultsFolder+sNameResultsZip+".zip");
	 	}
	 	else
	 	{
	 		fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "Problem doing the zip folder. The results will not send it by e-mail, if the case.");
	 	}
	}
	else if( _WIN32)
	{
		if(system("cd "+sNameResultsFolder+" && zip -r "+sNameResultsZip + " *.txt") == 0)
		{
	 		delay(FWBM_INTERBENCHMARK_DELAY);
	 		fwBenchmark_Util_sendEmail( dsListRecipients, "fwBenchmark - "+sDateTimePrefixReports, "", sNameResultsFolder+sNameResultsZip+".zip");
	 	}
	 	else
	 	{
	 		fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "Problem doing the zip folder. The results will not send it by e-mail, if the case.");
	 	}
	}
	else
	{
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Function not implemented for this OS.");
	}
 	
 	//End
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "***The reports are located in "+getPath(DATA_REL_PATH)+FWBM_RESULTS_FOLDER+"***");
 }
 
 
 