#uses "fwInstallation.ctl"
#uses "fwBenchmark/fwBenchmark_Util.ctl"
#uses "fwBenchmark/fwBenchmark_Const.ctl"

/*
 * 
 * This library contains the functions related
 * to the identification of the data server hosting the benchmark
 * 
 * jean-charles.tournier@cern.ch
 * CERN - EN/ICE/SCD
 * 
 * July 9th, 2015
 * 
 */
 
 
 /** 
 * 
 * Returns a dyn_string containing the characteristics of the machine
 * on which the benchmark is executed
 * 
 * @return value of type 'dyn_string'
 */
 dyn_string fwBenchmark_Machine_getCharacteristics()
 {
 	dyn_string dsCharac;
 	
 	dynAppend(dsCharac, FWBM_NEW_SECTION);
 	dynAppend(dsCharac, "Hostname:"+fwBenchmark_Machine_getHostname());
 	
 	int iNumCore;
  	string sProcName;
  	fwBenchmark_Machine_getProcessorDetails( iNumCore, sProcName);
  	dynAppend(dsCharac, "Number of cores:"+iNumCore);
  	dynAppend(dsCharac, "Processor:"+sProcName);
  
  	dynAppend(dsCharac, "Operating system:"+fwBenchmark_Machine_getOsGeneral());
  	dynAppend(dsCharac, "Operating system details:"+fwBenchmark_Machine_getOsDetails());
  
 	
  	dynAppend(dsCharac, "WinCC OA Version:"+fwBenchmark_Machine_getPvssVersion());
  
  	dyn_string dsListPatches = fwBenchmark_Machine_getListPatches();
  	int iNumPatches = dynlen(dsListPatches);
  	for( int i=1; i<=iNumPatches; i++)
  	{
  		if ( dsListPatches[i] != "")
  		{
  			dynAppend(dsCharac, "Patch "+i+" installed:"+dsListPatches[i]);
  		}
  	}
 
 	dynAppend(dsCharac, "Project name:"+fwBenchmark_Machine_getCurrentProject());
 	dynAppend(dsCharac, "fwBenchmark component version:"+fwInstallation_getComponentVersion("fwBenchmark"));
 	dynAppend(dsCharac, "Executing manager type:"+fwBenchmark_Machine_getCurrentManager()); 	
 	
 	return dsCharac;
 }
 
 
 /** 
 * Get basic hostname
 * @return value of type 'string'
 */
 string fwBenchmark_Machine_getHostname()
 {
 	return getHostname();
 }
 
 /** 
 *	Get the pvss version 
 * @return value of type 'string'
 */
 string fwBenchmark_Machine_getPvssVersion()
 {
 	return VERSION_DISP;
 }
 
 /** 
 *	Get the pvss version in numeric form 
 * @return value of type 'string'
 */
 string fwBenchmark_Machine_getPvssVersionNumeric()
 {
 	return (string)VERSION_NUMERIC;
 }
 
 /** 
 *	Get the OS in general 
 * @return value of type 'string'
 */
 string fwBenchmark_Machine_getOsGeneral()
 {
 	string sOs;
 	if ( _UNIX)
 	{
 		sOs = "Linux/Unix";	
 	} else if( _WIN32)
 	{
 		sOs = 	"Windows";
 	} else if( _SOLARIS)
 	{
 		sOs = "Solaris";	
 	} else
 	{
 		sOs = "Unknown";	
 	}
 	
 	return sOs;
 }
 
 /** 
 * Get the detail of the OS, which is given by 'uname' in linux
 * @return value of type 'string'
 */
 string fwBenchmark_Machine_getOsDetails()
 {
 	string sDetails;
 	
 	if ( _UNIX)
 	{
 		string sFileName = tmpnam();
 		system("uname -s -r -v -m -p -i -o > "+sFileName);
 		dyn_string dsContent = fwBenchmark_Util_readFile(sFileName);
 		if ( dynlen(dsContent) > 0)
 		{
 			sDetails = dsContent[1];		
 		}
 		system("rm "+sFileName);
 	}
 	else if( _WIN32)
 	{
		
		string BuildNumber, Caption, CSDVersion, Manufacturer,OSArchitecture,Version;
		
		fwBenchmark_Machine_getOsInformation("BuildNumber",BuildNumber);	
		fwBenchmark_Machine_getOsInformation("Caption",Caption);
		fwBenchmark_Machine_getOsInformation("CSDVersion",CSDVersion);	
		fwBenchmark_Machine_getOsInformation("Manufacturer",Manufacturer);
		fwBenchmark_Machine_getOsInformation("OSArchitecture",OSArchitecture);		
		fwBenchmark_Machine_getOsInformation("Version",Version);
			
		//Add to the string
		sDetails = 	Caption + " " +
					Version + " " +
					CSDVersion + " " +
					"Build " + BuildNumber + " " +
					OSArchitecture + " " +
					Manufacturer;
					
//		//Another way to get it: (systeminfo command. Slowier and necessary string functions to show a nice result)				
// 		string sFileName = tmpnam();
// 		system("systeminfo > "+sFileName);
// 		dyn_string dsContent = fwBenchmark_Util_readFile(sFileName);
// 		if ( dynlen(dsContent) > 0)
// 		{
//
// 			string osName, osVersion, osManufacturer, osConfig, osBuild;
// 			
// 			//"OS Name:                   Microsoft Windows 7 Enterprise "
// 			osName = dsContent[3];
//			osName=substr(osName, strpos(osName,":") + 1);	//Name without before to :
//			osName=strltrim(osName," ");		//Name without spaces
//
//			//...
// 			
// 			sDetails = 	osName + " " +
//			 			osVersion + " " +
//			 			osManufacturer + " " +
//			 			osConfig + " " +
//			 			osBuild;	
// 		}
// 		
// 		system("rm "+sFileName);

 	}
 	else
 	{
 		sDetails = "Not available";
 	}
 	
 	return sDetails;
 }
 
 
 /** 
 *	Get the list of patches installed on the machine 
 * @return value of type 'dyn_string'
 */
 dyn_string  fwBenchmark_Machine_getListPatches()
 {
 	dyn_string dsDetails;
 	
 	if ( _UNIX)
 	{
 		string sFileName = tmpnam();
 		system("ls "+PVSS_PATH+"Readme* > "+sFileName);
 		dsDetails = fwBenchmark_Util_readFile(sFileName);
 		if ( dynlen(dsDetails) < 1)
 		{
 			dsDetails[1] = "Could not be retrieved";	
 		} 		
 		dynRemove(dsDetails, dynlen(dsDetails));
 		system("rm "+sFileName);
 	}
 	else if( _WIN32)
 	{
		string sFileName = tmpnam();
		string pvsspath = PVSS_PATH;
		strreplace(pvsspath, "/", "\\"); //We have to change the / to \ on Windows
		
		dyn_string dsDetailsRaw;
		system("dir "+pvsspath+"Readme* > "+sFileName);
		dsDetailsRaw = fwBenchmark_Util_readFile(sFileName);
		if ( dynlen(dsDetailsRaw) < 1)
		{
			dsDetailsRaw[1] = "Could not be retrieved";	
			return dsDetailsRaw;
		}
		
		//On windows, we would like to have from the sixth element to the last 4 elements
		int indxj = 1;
		for(int i=6;i<=dynlen(dsDetailsRaw) - 4;i++)
		{
			dsDetails[indxj] = dsDetailsRaw[i]; 
			indxj++;
		}
		system("rm "+sFileName);
 	}
 	else
 	{
 		dsDetails[1] = "Not available";
 	} 	
 	return dsDetails;
 }
 
 /** 
 *	Get the number of processor as well as the name of the first core found 
 * @return value of type void
 */
 void fwBenchmark_Machine_getProcessorDetails( int &iNumCore, string &sProcName)
 {
 	dyn_string dsDetails;
 	
 	if ( _UNIX)
 	{
 		string sFileName = tmpnam();
 		system("cat /proc/cpuinfo | grep name  | cut -d':' -f2 > "+sFileName);
 		dsDetails = fwBenchmark_Util_readFile(sFileName);
 		iNumCore = dynlen(dsDetails)-1;
 		sProcName = "";
 		if ( dynlen(dsDetails) >= 1)
 		{
 			sProcName = dsDetails[1];	
 		}  		
 		system("rm "+sFileName);
 	}
  	else if( _WIN32)
 	{
		//Processor name
		string _sProcName = "";
		fwBenchmark_Machine_getCpuHardwareInformation("Name",_sProcName);
		sProcName = _sProcName;
		
		//Cores
		string NumberOfCores = "";
		fwBenchmark_Machine_getCpuHardwareInformation("NumberOfCores",NumberOfCores);
		iNumCore = (int) NumberOfCores;
 	}
 	else
 	{
 		iNumCore = -1;
 		sProcName = "Not available";
 	}
 	
 	return;
 }
 
 
 /** 
 * Get the hardware information through of commands (wmic os get <feature>)
 * 
 * Possible features: BootDevice, BuildNumber, BuildType, Caption, CodeSet, CountryCode, CreationClassName, CSCreationClassName,
 * CSDVersion, CSName, CurrentTimeZone, DataExecutionPrevention_32BitApplications, DataExecutionPrevention_Available,
 * DataExecutionPrevention_Drivers, DataExecutionPrevention_SupportPolicy, Debug, Description, Distributed, EncryptionLevel
 * ForegroundApplicationBoost, FreePhysicalMemory, FreeSpaceInPagingFiles, FreeVirtualMemory, InstallDate, LargeSystemCache
 * LastBootUpTime, LocalDateTime, Locale, Manufacturer, MaxNumberOfProcesses, MaxProcessMemorySiz, MUILanguages, Name,
 * NumberOfLicensedUsers, NumberOfProcesses, NumberOfUsers, OperatingSystemSKU, Organization, OSArchitecture, OSLanguage, OSProductSuite,
 * OSType, OtherTypeDescription, PAEEnabled, PlusProductID, PlusVersionNumber, Primary, ProductType, RegisteredUser, SerialNumber
 * ServicePackMajorVersion, ServicePackMinorVersion, SizeStoredInPagingFiles, Status, SuiteMask, SystemDevice, SystemDirectory,
 * SystemDrive, TotalSwapSpaceSize, TotalVirtualMemorySize, TotalVisibleMemorySize, Version, WindowsDirectory.
 * 
 * @param feature	(string)	IN String name of the feature to know
 * @param info	(string)	IN/OUT Information gained in 'echo' and converted in a string
 * @return value of type 'int'
 */
int fwBenchmark_Machine_getOsInformation(string feature , string &info) 
{
	string command;
	command = "cmd.exe /c @for /f \"skip=2 tokens=2 delims==\" %p in ('wmic os get " + feature + " /format:list') do @echo %p";
	if(_fwInstallation_getStringFromSystemCommand(command , info , true) != 0)
	{
		fwInstallation_throw("Couldn't get CPU information" , "ERROR" , 10);
		return -1;
	}
	return 0; 
}


/** 
 * Get the hardware information through of commands (wmic cpu get <feature>)
 * 
 * Possible features: AddressWidth, Architecture, Availability, Caption, ConfigManagerErrorCode, ConfigManagerUserConfig, CpuStatus,
 * CreationClassName, CurrentClockSpeed, CurrentVoltage, DataWidth, Description, DeviceID, ErrorCleared, ErrorDescription, ExtClock,
 * Family, InstallDate, L2CacheSize, L2CacheSpeed, L3CacheSize, L3CacheSpeed, LastErrorCode, Level, LoadPercentage, Manufacturer,
 * MaxClockSpeed, Name, NumberOfCores, NumberOfLogicalProcessors, OtherFamilyDescription, PNPDeviceID, PowerManagementCapabilities,
 * PowerManagementSupported, ProcessorId, ProcessorType, Revision, Role, SocketDesignation, Status, StatusInfo, Stepping,
 * SystemCreationClassName, SystemName, UniqueId, UpgradeMethod, Version, VoltageCaps
 * 
 * @param feature	(string)	IN String name of the feature to know
 * @param info	(string)	IN/OUT Information gained in 'echo' and converted in a string
 * @return value of type 'int'
 */
int fwBenchmark_Machine_getCpuHardwareInformation(string feature , string &info) 
{
	string command;
	command = "cmd.exe /c @for /f \"skip=2 tokens=2 delims==\" %p in ('wmic cpu get " + feature + " /format:list') do @echo %p";
	if(_fwInstallation_getStringFromSystemCommand(command , info , true) != 0)
	{
		fwInstallation_throw("Couldn't get CPU information" , "ERROR" , 10);
		return -1;
	}
	return 0; 
}

 
 /** 
 *	Return the name of the manager executing the benchmark 
 * @return value of type 'string'
 */
 string fwBenchmark_Machine_getCurrentManager()
 {
 	int iManType = myManType();
 	string sManager = "Unknown";
 	
 	if ( iManType == UI_MAN)
 	{
 		sManager = "UI";
 	} else if( iManType == CTRL_MAN)
 	{
 		sManager = "CTRL";
 	}
 	
 	return sManager;
 }
 
 /** 
 *	Simply returns the name of the project executing the benchmark 
 * @return value of type 'string'
 */
 string fwBenchmark_Machine_getCurrentProject()
 {
 	return PROJ;
 }