#uses "fwBenchmark/fwBenchmark_ConstUI.ctl"

/*
 * 
 * Functions related to the benchmark panels
 * 
 * 
 * jean-charles.tournier@cern.ch
 * CERN - EN/ICE/SCD
 * July 8th, 2015
 * 
 */
 
 /** 
 * 
 * Check that the report file is correctly specified
 * 
 * @return value of type 'bool' if the report file is specified or can be overwritten, false otherwise
 */
 bool fwBenchmark_UI_isReportFileOk()
 {
 	shape shapeFileName = getShape(FWBM_REPORT_FILE_TXT);
  	string sFileName = shapeFileName.text;
  
  	if (sFileName == "")
  	{
    	dyn_float dfR;
    	dyn_string dsR;
    	ChildPanelOnReturn( "vision/MessageWarning2", "Warning", makeDynString("$1:No file has been specified to save the report.\nContinue?","$2:Yes","$3:No"), -1, -1, dfR, dsR);
    	return ((int)dfR[1])  == 1;
  	}
  
  	if ( isfile(sFileName))
  	{
    	dyn_float dfR;
    	dyn_string dsR;
    	ChildPanelOnReturn( "vision/MessageWarning2", "Error", makeDynString("$1:Replace existing report file?","$2:Yes","$3:No"), -1, -1, dfR, dsR);
    	return ((int)dfR[1])  == 1;
  	}
  
  	return true;
 }
 
 
 /** 
 * 
 * Update the log table according to the level required
 * which is specified by the user with the check boxes
 * 
 * @param iLogLevel	(int)	IN	: 0 is no messages, 1 is info, 2 is warning, 4 is error. Any sum can be passed as parameters (e.g. 1+4 for info + error)
 * @return value of type 'void'
 */
 void fwBenchmark_UI_filterLogTable( int iLogLevel)
 {
 	bool bInfo = iLogLevel & 0x1;
 	bool bWarning = iLogLevel & 0x2;
 	bool bError = iLogLevel & 0x4;
 	
 	shape shTableLog = getShape(FWBM_LOG_TABLE);
 	int iNumRow = shTableLog.lineCount;
 	
 	for( int i=1; i<=iNumRow; i++)
 	{ 		
 		string sType = shTableLog.cellValueRC( i-1, "type");
 		if ( sType == "INFO" && bInfo ||
 			 sType == "WARNING" && bWarning  ||
 			 sType == "ERROR" && bError
 			)
 		{
 			shTableLog.showRow( i-1);
 		} else
 		{
 			shTableLog.hideRow( i-1);
 		}
 	}
 	
 }