 #uses "fwBenchmark/fwBenchmark_Const.ctl"
 #uses "fwBenchmark/fwBenchmark_Util.ctl"
/*
 * This library contains the support/common functions
 * to set and get attributes functions
 * 
 * alejandro.herrero.de.campos@cern.ch
 * CERN - EN/ICE/SCD
 * September 3th, 2015
 * 
 */
 
private const int _FWBM_ALERTHDL_NUMATTRIBUTES = 1400;	//Ensuring that is a multiple of 14
private const int _FWBM_ALERTHDL_BUNCHSTEP = 14;		//Always in bunches by 14 per 14


/** 
 * This function returns the number of DPEs to considerate depending
 * the configuration.
 * @param sConfiguration	(string)	IN 
 * @param iNumDp	(int)	IN 
 * @param iNumAttributes	(int)	IN/OUT 
 * @return value of type 'int'
 */
int fwBenchmark_UtilSetGet_DpsToHandle(string sConfiguration, int &iNumAttributes, int &iNumDp, int &iBunchStep)
{
	int iConfigLength = 0;
	
 	//Get the names depending of the configuration
 	switch(sConfiguration)
	{
		case FWBM_CONFIG_ORIGINALVALUE:
			iNumDp = iNumAttributes;
			break;
			
		case FWBM_CONFIG_DRIVER_SIM:
		case FWBM_CONFIG_DRIVER_S7:
		case FWBM_CONFIG_DRIVER_MODBUS:
		case FWBM_CONFIG_DRIVER_IEC:
			iConfigLength = dynlen(FWBM_DEVICE_CONFIG_NAME);
			if(iNumAttributes % iConfigLength == 0)
				iNumDp = iNumAttributes / iConfigLength;
			else
				iNumDp = iNumAttributes / iConfigLength + 1;
			break;
			
		case FWBM_CONFIG_ARCHIVE:
			iConfigLength = dynlen(FWBM_ARCHIVE_CONFIG_NAME);
			if(iNumAttributes % iConfigLength == 0)
				iNumDp = iNumAttributes / iConfigLength;
			else
				iNumDp = iNumAttributes / iConfigLength + 1;
			break;
		
		case FWBM_CONFIG_ALERTHDL:
			iConfigLength = dynlen(FWBM_ALERTHDL_CONFIG_NAME);
			
			iNumAttributes = _FWBM_ALERTHDL_NUMATTRIBUTES;		
			iBunchStep = _FWBM_ALERTHDL_BUNCHSTEP;
			
			iNumDp = iNumAttributes / iConfigLength;
			break;
			
		default:
			iNumDp = 0;
			return FWBM_PVSS_ERROR;	
	}
	
 	return FWBM_PVSS_SUCCESS;
}

/** 
 * 
 * @param sConfiguration	(string)	IN 
 * @param dsConfigurationNames	(dyn_string)	IN/OUT 
 * @param daConfigurationValues	(dyn_anytype)	IN/OUT 
 * @return value of type 'int'
 */
int fwBenchmark_UtilSetGet_GetNameValueConfiguration(string sConfiguration, dyn_string &dsConfigurationNames, dyn_anytype &daConfigurationValues)
{		
	
	dynClear(dsConfigurationNames);
	dynClear(daConfigurationValues);
	
	switch(sConfiguration)
	{
		case FWBM_CONFIG_ORIGINALVALUE:
			dsConfigurationNames = FWBM_ORIGINAL_CONFIG_NAME;
			daConfigurationValues = FWBM_ORIGINAL_CONFIG_VALUE;
			break;
			
		case FWBM_CONFIG_DRIVER_SIM:
			dsConfigurationNames = FWBM_DEVICE_CONFIG_NAME;
			daConfigurationValues = FWBM_SIM_CONFIG_VALUE;
			break;
			
		case FWBM_CONFIG_DRIVER_S7:
			dsConfigurationNames = FWBM_DEVICE_CONFIG_NAME;
			daConfigurationValues = FWBM_S7_CONFIG_VALUE;
			break;
			
		case FWBM_CONFIG_DRIVER_MODBUS:
			dsConfigurationNames = FWBM_DEVICE_CONFIG_NAME;
			daConfigurationValues = FWBM_MOD_CONFIG_VALUE;
			break;
			
		case FWBM_CONFIG_DRIVER_IEC:
			dsConfigurationNames = FWBM_DEVICE_CONFIG_NAME;
			daConfigurationValues = FWBM_IEC_CONFIG_VALUE;
			break;
			
		case FWBM_CONFIG_ARCHIVE:
			dsConfigurationNames = FWBM_ARCHIVE_CONFIG_NAME;
			daConfigurationValues = FWBM_ARCHIVE_CONFIG_VALUE;
			break;
			
		case FWBM_CONFIG_ALERTHDL:
			dsConfigurationNames = FWBM_ALERTHDL_CONFIG_NAME;
			daConfigurationValues = FWBM_ALERTHDL_CONFIG_VALUE;
			break;
			
		default:
			return FWBM_PVSS_ERROR;		
	}
	
	return FWBM_PVSS_SUCCESS;
}
 
/** 
 * Initial Conditions of the Attributes, i.e., if there is some configuration which needs launch by bunches of X or multiple of X.
 * @param sConfiguration	(string)	IN 
 * @param iNumAttributes	(int)	IN 
 * @param iBunchStep	(int)	IN 
 * @param iBunchSize	(int)	IN/OUT 
 * @return value of type 'int'
 */
int fwBenchmark_UtilSetGet_InitialConditionsOfAttributes(string sConfiguration, int iNumAttributes, int iBunchStep, int &iBunchSize)
{
	//Check the bunchstep size
	if(iBunchStep > iNumAttributes)
	{
		 fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "The bunch step cannot be higher of the attributes number");
		 return FWBM_PVSS_ERROR;
	}
	
	//1) There are some configurations which can not start with 1 attribute, they need start with the number of there attributes.
	//2) There are configurations which must be multiple of their number of attributes, and launch by bunches of number of attributes
	if(sConfiguration == FWBM_CONFIG_ALERTHDL)
	{
		int iAttributesNumberOfEachConfiguration = dynlen(FWBM_ALERTHDL_CONFIG_NAME);
		//1)
		iBunchSize = iAttributesNumberOfEachConfiguration;
		
		//2)
		if(iNumAttributes % iAttributesNumberOfEachConfiguration != 0)
		{
			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "The total number of attributes to be set/get in " 
				+ FWBM_CONFIG_ALERTHDL + " configuration must be multiple of the attributes number. In this case, multiple of "
				+ iAttributesNumberOfEachConfiguration +" elements.");
			return FWBM_PVSS_ERROR;
		}
		
		if(iBunchStep % iAttributesNumberOfEachConfiguration!= 0)
		{
			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "The bunch step of " 
				+ FWBM_CONFIG_ALERTHDL + " must be multiple of the attributes number. In this case, multiple of "
				+ iAttributesNumberOfEachConfiguration +" elements.");
			return FWBM_PVSS_ERROR;
		}

	}
	else
	{
		iBunchSize = 1;
	}
	return FWBM_PVSS_SUCCESS;
}


 /** 
 * Built correctly the vector to write into report
 * @param index	(int)	IN 
 * @param ddaResultIter	(dyn_dyn_float)	IN 
 * @param fElapsedTime	(float)	IN 
 * @param dfBunchSize	(dyn_float)	IN/OUT 
 * @param ddfTotalTimePerBunch	(dyn_dyn_float)	IN/OUT 
 * @param dfExecutionTime	(dyn_float)	IN/OUT 
 * @return value of type 'int'
 */
 int fwBenchmark_UtilSetGet_BuildVectorsToWrite(int index, dyn_dyn_float ddaResultIter, float fElapsedTime,
 													dyn_float &dfBunchSize, dyn_dyn_float &ddfTotalTimePerBunch,dyn_float &dfExecutionTime)
 {
 	//First iteration
	if(index == 1)
	{
		//Complete the bunches size vector. It's the same for all the iterations (We can choose from the first)
		const int iResultIterLength = dynlen(ddaResultIter);
		for(int j=1;j<=iResultIterLength;j++)
			dfBunchSize[j] = ddaResultIter[j][1];
	}
	
	//Complete the ddfTotalTimePerBunch vector
	//about the time elapsed per bunches per iteration (dyn_dyn_float)
	const int iResultIterLength = dynlen(ddaResultIter);
 	for(int j=1;j<=iResultIterLength;j++)
		ddfTotalTimePerBunch[index][j] = ddaResultIter[j][2];
	
	//Complete the vector about the time elapsed of the test per iteration
 		dfExecutionTime[index] = fElapsedTime;
 	return FWBM_PVSS_SUCCESS;
 }
 
/** 
 * Generate the report at the end of the benchmark
 * 
 * @param iNumDp	(int)	IN number of DP on which the function has been performed 
 * @param iBunchStep	(int)	IN difference in size between two bunches
 * @param sReportName	(string)	IN name of the benchmark header. Normally, dpGet or dpSet + Configuration
 * 									(Original value, S7 Device, Modbus, Archive...)
 * @param dfBunchSize	(dyn_float)	IN array with the bunches size (the same in all iterations)
 * @param ddfTotalTimePerBunch	(dyn_dyn_float)	IN two dimensions array with the first column being the iterations
 * 											and the second the time taken to perform the function
 * @param dfExecutionTime	(dyn_float)	IN array of the time to perform the test per iteration
 * @param sFileName	(string)	IN name of the file in which the report will be stored
 * @return value of type 'int' FWBM_PVSS_SUCCESS in case of success, FWBM_PVSS_ERROR otherwise
 */
int fwBenchmark_UtilSetGet_SaveReport(int iNumAttributes, int iNumDp , int iBunchStep, string sConfiguration, string sReportName, dyn_float dfBunchSize, dyn_dyn_float ddfTotalTimePerBunch,dyn_float dfExecutionTime, string sFileName) 
{
	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Generating the report...");
	
	dyn_string dsContent;
	dyn_string dsCommon = fwBenchmark_Util_getReportCommon( sReportName + " (Total number of attributes: "+iNumAttributes+" in "+iNumDp+" DPEs. Bunch increment: "+iBunchStep+"))");
 	
 	//Calculate
 	dyn_dyn_float ddfMinMaxExecTime;
 	dyn_float dfMinMaxExecTimeAbsolute;
 	fwBenchmark_UtilSetGet_CalculateStatisticalValues(ddfTotalTimePerBunch,dfBunchSize,
 		ddfMinMaxExecTime,dfMinMaxExecTimeAbsolute
 	);
 	
	//Append to a vector to write the report
	_fwBenchmark_UtilSetGet_AddToVectorContent(dsCommon, ddfTotalTimePerBunch, dfBunchSize, sConfiguration, ddfMinMaxExecTime, dfMinMaxExecTimeAbsolute, dfExecutionTime,
		dsContent
	);
	
	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Report generation completed. Saving report to file.");

 	return fwBenchmark_Util_writeFile( sFileName, dsContent);
}

/** 
 * Calculate some statistical values as the minimum and maximum elapsed time per round
 * 
 * @param ddfTotalTimePerBunch	(dyn_dyn_float)	IN 
 * @param dfBunchSize	(dyn_float)	IN 
 * @param ddfMinMaxExecTime	(dyn_dyn_float)	IN/OUT 
 * @param dfMinMaxExecTimeAbsolute	(dyn_float)	IN/OUT 
 * @return value of type 'int'
 */
int fwBenchmark_UtilSetGet_CalculateStatisticalValues(dyn_dyn_float ddfTotalTimePerBunch, dyn_float dfBunchSize,
															dyn_dyn_float &ddfMinMaxExecTime, dyn_float &dfMinMaxExecTimeAbsolute)
{
	//Calculate the minimum and maximum per iteration and save to a vector, together with the index
 	dynClear(ddfMinMaxExecTime);
 	_fwBenchmark_UtilSetGet_MinAndMaxWithIndex(ddfTotalTimePerBunch,dfBunchSize,	ddfMinMaxExecTime);
 	
	//Minimum and maximum absolutes, with the previously vector calculated input
	//and returns the maximum and minimum with the bunch size index and with the iteration number which was found
	dynClear(dfMinMaxExecTimeAbsolute);
	_fwBenchmark_UtilSetGet_MinAndMaxVectorsInput(ddfMinMaxExecTime, dfMinMaxExecTimeAbsolute);
 	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Append the information about elapsed times and statistical values to the vector Content to write in a file
 * 
 * @param dsCommon	(dyn_string)	IN 
 * @param ddfTotalTimePerBunch	(dyn_dyn_float)	IN 
 * @param dfBunchSize	(dyn_float)	IN 
 * @param ddfMinMaxExecTime	(dyn_dyn_float)	IN 
 * @param dfMinMaxExecTimeAbsolute	(dyn_float)	IN 
 * @param dfExecutionTime	(dyn_float)	IN 
 * @param dsContent	(dyn_string)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_UtilSetGet_AddToVectorContent(dyn_string dsCommon, dyn_dyn_float ddfTotalTimePerBunch, dyn_float dfBunchSize,
															string sConfiguration,
															dyn_dyn_float ddfMinMaxExecTime, dyn_float dfMinMaxExecTimeAbsolute, dyn_float dfExecutionTime,
															dyn_string &dsContent)
{
	dynClear(dsContent);
	// Write
 	dyn_string dsLogContent;
 	_fwBenchmark_UtilSetGet_AddToVectorContentGeneral(dsCommon,	dsContent);
 	_fwBenchmark_UtilSetGet_AddConfiguration(sConfiguration, dsContent);
 	_fwBenchmark_UtilSetGet_AddToVectorContentEachRound(ddfTotalTimePerBunch,dfBunchSize,dfExecutionTime, ddfMinMaxExecTime,	dsContent);
	_fwBenchmark_UtilSetGet_AddToVectorContentTestElapsedTime(dfExecutionTime,	dsContent,dsLogContent);
	_fwBenchmark_UtilSetGet_AddToVectorContentMinAndMax(dfMinMaxExecTimeAbsolute,	dsContent, dsLogContent);										
	fwBenchmark_UtilSetGet_WriteLogs(dsLogContent);
	return FWBM_PVSS_SUCCESS;
}
 
 /** 
 * Calculate the minimum and maximum per iteration and save to a vector, together with the index
 * @param ddfTotalTimePerBunch	(dyn_dyn_float)	IN 
 * @param dfBunchSize	(dyn_float)	IN 
 * @param ddfMinMaxExecTime	(dyn_dyn_float)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_UtilSetGet_MinAndMaxWithIndex(dyn_dyn_float ddfTotalTimePerBunch, dyn_float dfBunchSize,
									dyn_dyn_float &ddfMinMaxExecTime)
{
	//Variables for the minimum and maximum partial
 	int iMinBunchSize = -1;
 	int iMaxBunchSize = -1;
 	float fMinExecTime = maxFLOAT();
 	float fMaxExecTime = -1;
 	
 	//Vectors to return (in ddfMinMaxExecTime)
 	dyn_float diMinBunchSize;
 	dyn_float diMaxBunchSize;
 	dyn_float dfMinExecTime;
 	dyn_float dfMaxExecTime;
 	dynClear(ddfMinMaxExecTime);
 	
 	//Minimum and maximum per iteration (relative)
 	const int iTotalTimePerBunchLength = dynlen(ddfTotalTimePerBunch); 
 	for(int i=1;i<=iTotalTimePerBunchLength;i++)			//i iteration
 	{
 		iMinBunchSize = -1;
	 	iMaxBunchSize = -1;
	 	fMinExecTime = maxFLOAT();
	 	fMaxExecTime = -1;
	 	const int iTotalTimePerBunchLengthCol = dynlen(ddfTotalTimePerBunch[i]); 
 		for(int j=1;j<=iTotalTimePerBunchLengthCol;j++)		//j bunches
 		{
 			float fExecTime = ddfTotalTimePerBunch[i][j];
			int iBunchSize = (int)dfBunchSize[j];
			
 			if ( fExecTime > fMaxExecTime)
		    {
		      fMaxExecTime = fExecTime;
		      iMaxBunchSize = iBunchSize;
		    }
		    
		    if ( fExecTime < fMinExecTime)
		    {
		      fMinExecTime = fExecTime;
		      iMinBunchSize = iBunchSize;
		    }	
 		}
 		
 	 	diMinBunchSize[i] = iMinBunchSize;
	 	diMaxBunchSize[i] = iMaxBunchSize;
	 	dfMinExecTime[i] = fMinExecTime;
	 	dfMaxExecTime[i] = fMaxExecTime;
 	}
 	
 	ddfMinMaxExecTime[1] = diMinBunchSize;
 	ddfMinMaxExecTime[2] = dfMinExecTime;
 	ddfMinMaxExecTime[3] = diMaxBunchSize;
 	ddfMinMaxExecTime[4] = dfMaxExecTime;
 	
	return FWBM_PVSS_SUCCESS;
}
 
/** 
 * Minimum and maximum absolutes, with the previously vector calculated input
 * and returns the maximum and minimum with the bunch size index and with the iteration number which was found
 * @param ddfMinMaxExecTime	(dyn_dyn_float)	IN 
 * @param dfMinMaxExecTimeAbsolute	(dyn_float)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_UtilSetGet_MinAndMaxVectorsInput(dyn_dyn_float ddfMinMaxExecTime,	dyn_float &dfMinMaxExecTimeAbsolute)
{
	//reset min and max variables
	int iMinBunchSizeAbsolute = -1;
 	int iMaxBunchSizeAbsolute = -1;
 	float fMinExecTimeAbsolute = maxFLOAT();
 	float fMaxExecTimeAbsolute = -1;
 	int iIterationMin = -1;
 	int iIterationMax = -1;
 	
 	dyn_float diMinBunchSize = ddfMinMaxExecTime[1];
 	dyn_float dfMinExecTime = ddfMinMaxExecTime[2];
 	
 	dyn_float diMaxBunchSize = ddfMinMaxExecTime[3];
 	dyn_float dfMaxExecTime = ddfMinMaxExecTime[4];
 	
 	const int iMinExecTimeLength = dynlen(dfMinExecTime); 
 	for (int i = 1; i <= iMinExecTimeLength; i++)
 	{		
		float fExecTimeMin = dfMinExecTime[i];
 	 	int iBunchSizeMin = (int)diMinBunchSize[i];
		if ( fExecTimeMin < fMinExecTimeAbsolute)
		{
		  fMinExecTimeAbsolute = fExecTimeMin;
		  iMinBunchSizeAbsolute = iBunchSizeMin;
		  iIterationMin = i;
		}
		
	 	float fExecTimeMax = dfMaxExecTime[i];
 	 	int iBunchSizeMax = (int)diMaxBunchSize[i];
		if ( fExecTimeMax > fMaxExecTimeAbsolute)
		{
		  fMaxExecTimeAbsolute = fExecTimeMax;
		  iMaxBunchSizeAbsolute = iBunchSizeMax;
		  iIterationMax = i;
		}
 	}
 	
 	dynClear(dfMinMaxExecTimeAbsolute); 
 	
 	dfMinMaxExecTimeAbsolute[1] = iMinBunchSizeAbsolute;
 	dfMinMaxExecTimeAbsolute[2] = fMinExecTimeAbsolute;
 	dfMinMaxExecTimeAbsolute[3] = iIterationMin;
 	
 	dfMinMaxExecTimeAbsolute[4] = iMaxBunchSizeAbsolute;
 	dfMinMaxExecTimeAbsolute[5] = fMaxExecTimeAbsolute;
 	dfMinMaxExecTimeAbsolute[6] = iIterationMax;

 	
 	return FWBM_PVSS_SUCCESS;
}

/** 
 * Append the general information about the machine
 * @param dsCommon	(dyn_string)	IN 
 * @param dsContent	(dyn_string)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_UtilSetGet_AddToVectorContentGeneral(dyn_string dsCommon,dyn_string &dsContent)
{
	//Write machine info
 	dynAppend(dsContent, dsCommon);
 	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Add the configuration parameters (The setting and getting attributes of each DPEs)
 * @param sConfiguration	(string)	IN 
 * @param dsContent	(dyn_string)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_UtilSetGet_AddConfiguration(string sConfiguration, dyn_string &dsContent)
{
	dyn_string dsConfigurationNames;
	dyn_anytype daConfigurationValues;
	
	//Link the attribute name with the value, depending of the configuration
	fwBenchmark_UtilSetGet_GetNameValueConfiguration(sConfiguration, 	dsConfigurationNames, daConfigurationValues);
	
	dynAppend(dsContent, FWBM_NEW_SECTION);
	dynAppend(dsContent, "Configuration to be Set or Get in the DPEs");
	dynAppend(dsContent, "Configuration Name:\n" + sConfiguration);
	dynAppend(dsContent, "Attributes:");
	const int dsConfigurationNamesLength = dynlen(dsConfigurationNames);
	for (int i = 1; i <= dsConfigurationNamesLength; i++) {
	     string sWrite = "[" + i + "] '" + dsConfigurationNames[i] + "' = " + daConfigurationValues[i];
	     dynAppend(dsContent, sWrite);     
	}
	
	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Add the information about elapsed time and bunches per round
 * @param ddfTotalTimePerBunch	(dyn_dyn_float)	IN 
 * @param dfBunchSize	(dyn_float)	IN 
 * @param dfExecutionTime	(dyn_float)	IN 
 * @param ddfMinMaxExecTime	(dyn_dyn_float)	IN 
 * @param dsContent	(dyn_string)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_UtilSetGet_AddToVectorContentEachRound(dyn_dyn_float ddfTotalTimePerBunch,dyn_float dfBunchSize, dyn_float dfExecutionTime,
																dyn_dyn_float ddfMinMaxExecTime,	dyn_string &dsContent)
{
	
	dyn_float diMinBunchSize = ddfMinMaxExecTime[1];
 	dyn_float dfMinExecTime = ddfMinMaxExecTime[2];
 	
 	dyn_float diMaxBunchSize = ddfMinMaxExecTime[3];
 	dyn_float dfMaxExecTime = ddfMinMaxExecTime[4];
	
 	const int iTotalTimePerBunchLength = dynlen(ddfTotalTimePerBunch); 
 	//Write information of each iteration
 	for (int i = 1; i <= iTotalTimePerBunchLength; i++)
 	{
		_fwBenchmark_UtilSetGet_AddToVectorContentElapsedTimePerBunchPerRound(i, ddfTotalTimePerBunch[i], dfBunchSize, dfExecutionTime[i], dsContent);
 		
 		dynAppend(dsContent, "Statistics");
		string sStatMax = "Max Elapsed Time: "+dfMaxExecTime[i]+" ( "+diMaxBunchSize[i]+" bunches)";  
		string sStatMin = "Min Elapsed Time: "+dfMinExecTime[i]+" ( "+diMinBunchSize[i]+" bunches)";
		dynAppend(dsContent, sStatMax);
		dynAppend(dsContent, sStatMin);
  	}
  	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Add the elapsed time per bunch and per round
 * @param iRound	(int)	IN 
 * @param dfTotalTimePerBunch	(dyn_float)	IN 
 * @param dfBunchSize	(dyn_float)	IN 
 * @param fExecutionTime	(float)	IN 
 * @param dsContent	(dyn_string)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_UtilSetGet_AddToVectorContentElapsedTimePerBunchPerRound(int iRound, dyn_float dfTotalTimePerBunch, dyn_float dfBunchSize, float fExecutionTime,
																	dyn_string &dsContent)
{
 	dynAppend(dsContent, FWBM_NEW_SECTION);
	string sRoundNum = "Round "+iRound+"\n";
	dynAppend(dsContent, sRoundNum);
 	dynAppend(dsContent, "Bunch size\tElapsed time");
 	const int iTotalTimePerBunchLengthCol = dynlen(dfTotalTimePerBunch); 
 	for(int j=1; j<=iTotalTimePerBunchLengthCol; j++)
 	{
 		float iBunchSize = dfBunchSize[j];	//it will be type int
		float fExecTime = dfTotalTimePerBunch[j];
		dynAppend(dsContent, iBunchSize+"\t\t"+fExecTime);	
 	}
 	string sExecutionTimeFormatted;
	sprintf(sExecutionTimeFormatted, "%.4f", fExecutionTime);
	string sTimeRound = "Time to complete the test: "+sExecutionTimeFormatted; 	
	dynAppend(dsContent, sTimeRound);
	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Add the data time to perform the test
 * @param dfExecutionTime	(dyn_float)	IN 
 * @param dsContent	(dyn_string)	IN/OUT 
 * @param dsLogContent	(dyn_string)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_UtilSetGet_AddToVectorContentTestElapsedTime(dyn_float dfExecutionTime, dyn_string &dsContent, dyn_string &dsLogContent)
{
	float fTotalTime = 0;
	const int iExecutionTimeLength = dynlen(dfExecutionTime); 
	for (int i = 1; i <= iExecutionTimeLength; i++)
	{
		fTotalTime = fTotalTime + dfExecutionTime[i];
	}
	//Write the elapsed time to perform the test
 	dynAppend(dsContent, FWBM_NEW_SECTION);
 	dynAppend(dsContent, "Elapsed time to perform the test\n");
 	string sStatAvg = "Average time to complete the test per iteration: " + dynAvg(dfExecutionTime);
 	string sTotalTime = "Total time used to complete the test: " + fTotalTime;
 	dynAppend(dsContent, sStatAvg); 
 	dynAppend(dsContent, sTotalTime);
 	
 	dynAppend(dsLogContent, sStatAvg);
 	dynAppend(dsLogContent, sTotalTime);
 	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Add the minimum and maximum values obtained by the CalculateStatisticalValues function
 * @param ddfMinMaxExecTimeAbsolute	(dyn_float)	IN 
 * @param dsContent	(dyn_string)	IN/OUT 
 * @param dsLogContent	(dyn_string)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_UtilSetGet_AddToVectorContentMinAndMax(dyn_float ddfMinMaxExecTimeAbsolute,
						dyn_string &dsContent, dyn_string &dsLogContent)
{
	
	int iMinBunchSizeAbsolute = (int)ddfMinMaxExecTimeAbsolute[1];
	float fMinExecTimeAbsolute = ddfMinMaxExecTimeAbsolute[2];
	int iIterationMin = (int)ddfMinMaxExecTimeAbsolute[3];
	
	int iMaxBunchSizeAbsolute = (int)ddfMinMaxExecTimeAbsolute[4];
	float fMaxExecTimeAbsolute = ddfMinMaxExecTimeAbsolute[5];
	int iIterationMax = (int)ddfMinMaxExecTimeAbsolute[6];
	
	//Write minimum and maximum absolute
	dynAppend(dsContent, FWBM_NEW_SECTION);
	dynAppend(dsContent, "Statistics");
	string sStatMax = "Max Elapsed Time Absolute: "
		+fMaxExecTimeAbsolute+" ( "+iMaxBunchSizeAbsolute+" bunches)" + ", Round " + iIterationMax;  
	string sStatMin = "Min Elapsed Time Absolute: "
		+fMinExecTimeAbsolute+" ( "+iMinBunchSizeAbsolute+" bunches)" + ", Round " + iIterationMin; 
	dynAppend(dsContent, sStatMax);
	dynAppend(dsContent, sStatMin); 
	
	string sText = "Statistics:";
	dynAppend(dsLogContent,sText);
	dynAppend(dsLogContent, sStatMax);
 	dynAppend(dsLogContent, sStatMin);
	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Write some information into the Logs panel
 * @param dsLogContent	(dyn_string)	IN 
 * @return value of type 'int'
 */
int fwBenchmark_UtilSetGet_WriteLogs(dyn_string dsLogContent)
{
	//Show some results in the log
	const int iLogContentLength = dynlen(dsLogContent);
	for (int i = 1; i <= iLogContentLength; i++)
	{
		fwBM_log( FWBM_INFO_LOG, __FUNCTION__, dsLogContent[i]); 
	}
	
	return FWBM_PVSS_SUCCESS;
}