#!/bin/sh

#Functions
DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
. $DIR/../libs/fwBenchmark/fwBenchmarkScript_Const.sh
. $DIR/../libs/fwBenchmark/fwBenchmarkScript_Sysbench.sh
. $DIR/../libs/fwBenchmark/fwBenchmarkScript_Hdparm.sh
. $DIR/../libs/fwBenchmark/fwBenchmarkScript_Util.sh  
. $DIR/../libs/fwBenchmark/fwBenchmarkScript_UtilTests.sh
. $DIR/../libs/fwBenchmark/fwBenchmarkScript_Save.sh

##### Main
echo $FWBM_SC_NEWSECTION

iNumArgs=$#

sDate="$(date +%Y_%m_%d-%H_%M_%S)"
sMail="NULL"
sBaseFileName="$DIR/Results$sDate"	#Default directory

#Check the options
if [ "$iNumArgs" == "0" ]; then

	echo $FWBM_SC_NEWSECTION
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "You have not entered any directory."
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "The results will be save in the default directory: $sBaseFileName"
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "You can specify the directory when you lunch the script."
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "You also can specify an email to send the results."
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Type \"$0 --help\" for more information"
	echo $FWBM_SC_NEWSECTION

elif [ "$iNumArgs" == "1" ]; then
	sOption=$1
	case "$sOption" in

	"-h" | "--help")		fwBenchmarkScript_UtilTests_DisplayHelp
							exit 1
							;;

	*)						fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option: $1"
							fwBenchmarkScript_UtilTests_DisplayHelp
							exit $FWBM_SC_ERROR
							;;
	esac

elif [ "$iNumArgs" == "2" ]; then
	sOption=$1
	sName=$2

	case "$sOption" in

	"-d" | "--directory")	sBaseFileName=$sName
 							sBaseFileName="$sBaseFileName$sDate"
 							fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "The following folder will be created and the results will be saved in it:"
 							fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "$sBaseFileName"
							;;

	"-m" | "--mail")		sMail=$sName
 							sBaseFileName="$DIR$sDate"
 							fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Reports will be sent to $sMail"
						 	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "You have not entered any directory."
							fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "The results will be save in the default directory,"
							fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "where the script has been launched: $sBaseFileName"
							;;

	*)						fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option: $sOption"
							fwBenchmarkScript_UtilTests_DisplayHelp
							exit $FWBM_SC_ERROR
							;;
	esac



elif [ "$iNumArgs" == "4" ]; then
	sOption1=$1
	sBaseFileName=$2
	sBaseFileName="$sBaseFileName$sDate"

	sOption2=$3
	sMail=$4

	case "$sOption1" in

	"-d" | "--directory")	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "The following folder will be created and the results will be saved in it:"
 							fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "$sBaseFileName"
							;;

	*)						fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option: $sOption1"
							fwBenchmarkScript_UtilTests_DisplayHelp
							exit $FWBM_SC_ERROR
							;;
	esac

	case "$sOption2" in

	"-m" | "--mail")	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Reports will be sent to $sMail"
						;;

	*)					fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option: $sOption2"
						fwBenchmarkScript_UtilTests_DisplayHelp
						exit $FWBM_SC_ERROR
						;;
	esac

else
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option"
	fwBenchmarkScript_UtilTests_DisplayHelp
	exit $FWBM_SC_ERROR
fi

#Check if the result folder exists and create otherwise
mkdir $sBaseFileName > /dev/null 2>&1
iErr=$?
if [ "$iErr" -ne "0" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "The directory does not exists and/or it is not possible to create."
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Maybe, trying to create directory and subdirectory at the same time. Stopping..."
	exit $FWBM_SC_ERROR
fi

##Running the main functions
##	HDPARM TESTS:
fwBenchmarkScipt_Hdparm_Launch $sBaseFileName $FWBM_SC_HDPARM_DIRECT
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Stopping the benchmark due to error."
	exit $FWBM_SC_ERROR
fi

fwBenchmarkScipt_Hdparm_Launch $sBaseFileName $FWBM_SC_HDPARM_CACHE
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Stopping the benchmark due to error."
	exit $FWBM_SC_ERROR
fi

##	SYSBENCH TESTS:
fwBenchmarkScript_Sysbench_Launch $sBaseFileName $FWBM_SC_SYSBENCH_CPU
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Stopping the benchmark due to error."
	exit $FWBM_SC_ERROR
fi

fwBenchmarkScript_Sysbench_Launch $sBaseFileName $FWBM_SC_SYSBENCH_FILEIO
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Stopping the benchmark due to error."
	exit $FWBM_SC_ERROR
fi

fwBenchmarkScript_Sysbench_Launch $sBaseFileName $FWBM_SC_SYSBENCH_THREADS
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Stopping the benchmark due to error."
	exit $FWBM_SC_ERROR
fi

fwBenchmarkScript_Sysbench_Launch $sBaseFileName $FWBM_SC_SYSBENCH_MUTEX
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Stopping the benchmark due to error."
	exit $FWBM_SC_ERROR
fi

fwBenchmarkScript_Sysbench_Launch $sBaseFileName $FWBM_SC_SYSBENCH_MEMORY
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Stopping the benchmark due to error."
	exit $FWBM_SC_ERROR
fi

#Completed
echo $FWBM_SC_NEWSECTION
fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "Benchmark Standard test completed."

#Zip in the results in a folder
sNameResultsZip="$sDate-fwStandardBenchmark"
command cd $sBaseFileName && zip $sNameResultsZip -name *.txt > /dev/null 2>&1
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Unable to compress the files"
	exit $FWBM_SC_ERROR
fi

#Send the results by email
if [ "$sMail" != "NULL" ]; then
	fwBenchmarkScript_Util_SendEmail $sMail "Standard Benchmarks" "" "$sBaseFileName/$sNameResultsZip.zip"
	iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error sending the e-mail. The results will not send by e-mail."
		exit $FWBM_SC_ERROR
	fi
fi

exit $FWBM_SC_SUCCESS
