#!/bin/sh

CURRENT_DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )

## Include libraries
. "$CURRENT_DIR/../libs/fwBenchmark/fwBenchmarkScript_Const.sh"
. "$CURRENT_DIR/../libs/fwBenchmark/fwBenchmarkScript_Util.sh"
. "$CURRENT_DIR/../libs/fwBenchmarkBootstrapper/fwBenchmarkBootstrapper_Util.sh"

## Variables 
sTemporalFolder="/tmp/"
sLogFileName=$sTemporalFolder"_fwBenchmarkBootstrapper_Install_And_Run.log"

sBootstrapperFolderBaseName="$CURRENT_DIR/../component/"
sBootstrapperFolderName=$sBootstrapperFolderBaseName"PVSSBootstrapper.r149715"
sBootstrapperZipFileName=$sBootstrapperFolderName".zip"		#https://svn.cern.ch/reps/en-ice-svn/trunk/tools/PVSS/Research/PVSSBootstrapper/

sBootstrapperPythonScriptPath="$CURRENT_DIR/../libs/fwBenchmarkBootstrapper/fwBenchmarkBootstrapper.py"
sStandardBenchmarksScriptPath="$CURRENT_DIR/../fwBenchmark/fwBenchmarkScriptStd.sh"
sStandardBenchmarksScriptResults="$CURRENT_DIR/../../Test/"

sProjectName="fwBenchmarkBootstrapperProject" 
sMail="NULL"

## Main
iNumArgs=$#

# Check the options
if [ "$iNumArgs" == "0" ]; then

	echo $FWBM_SC_NEWSECTION
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "You have not entered any project name"
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "The project name by default is $sProjectName"
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "You can specify the directory when you lunch the script."
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "You also can specify an email to send the results."
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Type \"$0 --help\" for more information"
	echo $FWBM_SC_NEWSECTION

elif [ "$iNumArgs" == "1" ]; then
	sOption=$1
	case "$sOption" in

	"-h" | "--help")		fwBenchmarkBoostrapper_Util_DisplayHelp
							exit $FWBM_SC_ERROR
							;;

	*)						fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option: $1"
							fwBenchmarkBoostrapper_Util_DisplayHelp
							exit $FWBM_SC_ERROR
							;;
	esac

elif [ "$iNumArgs" == "2" ]; then
	sOption=$1
	sName=$2

	case "$sOption" in

	"-p" | "--project")		sProjectName=$sName
 							fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "The project name is $sProjectName"
							;;

	"-m" | "--mail")		sMail=$sName
 							fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Reports will be sent to $sMail"
						 	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "You have not entered any project name."
							fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "The project name by default is $sProjectName"
							;;

	*)						fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option: $sOption"
							fwBenchmarkBoostrapper_Util_DisplayHelp
							exit $FWBM_SC_ERROR
							;;
	esac



elif [ "$iNumArgs" == "4" ]; then
	sOption1=$1
	sProjectName=$2

	sOption2=$3
	sMail=$4

	case "$sOption1" in

	"-p" | "--project")	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "The project name is $sProjectName"
							;;

	*)						fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option: $sOption1"
							fwBenchmarkBoostrapper_Util_DisplayHelp
							exit $FWBM_SC_ERROR
							;;
	esac

	case "$sOption2" in

	"-m" | "--mail")	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Reports will be sent to $sMail"
						;;

	*)					fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option: $sOption2"
						fwBenchmarkBoostrapper_Util_DisplayHelp
						exit $FWBM_SC_ERROR
						;;
	esac

else
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option"
	fwBenchmarkBoostrapper_Util_DisplayHelp
	exit $FWBM_SC_ERROR
fi

# Init
fwBenchmarkScript_Util_AdminPermission
iAdminPermission=$?	#Save the return value. If admin returns 0, else returns 1.
if [ "$iAdminPermission" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "You must have administrator permission to run the Benchmark. Stopping..."
	exit $FWBM_SC_ERROR
fi
fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Starting the Bootstrapper Installer"
fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "If any error exists, there is a logger in $sLogFileName"
fwBenchmarkScript_Util_Logs $FWBM_SC_WARNING_LOG "The Bootstrapper, when it is installed, it will stop all the WinCC-OA projects."
fwBenchmarkScript_Util_Logs $FWBM_SC_WARNING_LOG "Do you want to continue [y/N]"

read sContinueBenchmark
if [ "$sContinueBenchmark" == "N" ] || [ "$sContinueBenchmark" == "n" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Stopping..."
	exit $FWBM_SC_ERROR
elif [ "$sContinueBenchmark" == "Y" ] || [ "$sContinueBenchmark" == "y" ]; then
	continue
else
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error: Unknown option"
	exit $FWBM_SC_ERROR
fi

# Insert a date in the log file
echo $FWBM_SC_NEWSECTION > $sLogFileName	#Overwrite the file. sudo, for the first operation which the scripts uses command line
iDate="$(date +%Y_%m_%d-%H_%M_%S)"
echo $iDate >> $sLogFileName
echo $FWBM_SC_NEWSECTION >> $sLogFileName

# Unzip the Bootstrapper. Delete the folder directory if existing (to avoid problems with overwriting)
cd $sBootstrapperFolderBaseName >> $sLogFileName
sudo rm -rf $sBootstrapperFolderName >> $sLogFileName 2>&1
fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Unzip the PVSSBootstrapper file in $sBootstrapperFolderName"
sudo unzip $sBootstrapperZipFileName -d $sBootstrapperFolderBaseName >> $sLogFileName
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with unzip command in $sBootstrapperZipFileName"
	exit $FWBM_SC_ERROR
fi

# Install Bootstrapper
fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Installing the Bootstrapper for WinCC-OA."
echo "Installing the Bootstrapper for WinCC-OA" >> $sLogFileName
cd $sBootstrapperFolderName >> $sLogFileName
sudo python setup.py install >> $sLogFileName 2>&1
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with the Bootstrapper installation."
	exit $FWBM_SC_ERROR
fi
fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The bootstrapper has been installed."

# Run the benchmark!
fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "Running the Bootstrapper python script for installing the components and Launch the fwBenchmark control with WinCC-OA."
sCommand="sudo python $sBootstrapperPythonScriptPath $sProjectName $sMail"
command $sCommand
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error running the bootstrapper and/or launching the fwBenchmark-WinCCOA"
	exit $FWBM_SC_ERROR
fi
fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The Bootstrapper of the fwBenchmark control with WinCC-OA has finished correctly."

# Run the standard benchmark!
fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "Running the script for Standard Benchmarks"
sCommand="$sStandardBenchmarksScriptPath -d $sStandardBenchmarksScriptResults -m $sMail"
command $sCommand
iErr=$?
if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error running the Standard Benchmarks"
	exit $FWBM_SC_ERROR
fi
fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The standard benchmarks has finished correctly."

# End
fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The Benchmark is complete. If you provided an e-mail, the results will be sent"

echo $FWBM_SC_NEWSECTION >> $sLogFileName
