#!/bin/sh

function fwBenchmarkScipt_Hdparm_Launch()
{
	local sBaseFileName=$1
	local sTestModeName=$2

	echo $FWBM_SC_NEWSECTION
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Starting the $FWBM_SC_HDPARM test"

	#Check if the benchmark is installed
	fwBenchmarkScript_UtilTests_Init $FWBM_SC_HDPARM
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Stopping the benchmark due to error."
		exit $FWBM_SC_ERROR
	fi

	#Run the tests
	fwBenchmarkScipt_Hdparm_Run $sBaseFileName $sTestModeName
	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		return $FWBM_SC_ERROR
	fi

	return $FWBM_SC_SUCCESS
}


function fwBenchmarkScipt_Hdparm_Run()
{
	local sBaseFileName=$1
	local sTestModeName=$2

	#Disk Number
	#return some name like /dev/sda1 - by default get the mounting point of the current script
	local sDiskName="$(df -P ${0}| awk 'NR==2' | awk -F " " '{print $1}')"	
	if [ -z "$sDiskName" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "No mounting point could be found. Stopping..."
		return $FWBM_SC_ERROR
	fi	

	#Ask for permission
	fwBenchmarkScript_Util_AdminPermission
	local iAdminPermission=$?	#Save the return value. If admin returns 0, else returns 1.
	if [ "$iAdminPermission" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "You must have administrator permission to run the $FWBM_SC_HDPARM test. Stopping..."
		return $FWBM_SC_ERROR
	fi

	#Launch test
	case "$sTestModeName" in
		$FWBM_SC_HDPARM_DIRECT)
 			fwBenchmarkScipt_Hdparm_Core_Direct $sBaseFileName $sDiskName
 			local iErr=$?
			if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
				return $FWBM_SC_ERROR
			fi
			;;

		$FWBM_SC_HDPARM_CACHE)
 			fwBenchmarkScipt_Hdparm_Core_Cache $sBaseFileName $sDiskName
 			local iErr=$?
			if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
				return $FWBM_SC_ERROR
			fi
			;;

		*)	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "The test $FWBM_SC_HDPARM is unknow"
 			return $FWBM_SC_ERROR
			;;
	esac

	return $FWBM_SC_SUCCESS
}

function fwBenchmarkScipt_Hdparm_Core_Direct()
{
	local sBaseFileName=$1
	local sDiskName=$2

	local sName="DeviceRead"

	#Report file name
	local iDate="$(date +%Y_%m_%d-%H_%M_%S)"
	local sFileReport=$iDate"-"$FWBM_SC_HDPARM"-"$sName".txt"
	local sFileReport=$sBaseFileName"/"$sFileReport
	local sFileReportTemp=$(mktemp)

	#Running test
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "Running the $FWBM_SC_HDPARM, Direct device read test on $sDiskName."
	local sCommand="sudo $FWBM_SC_HDPARM -$FWBM_SC_HDPARM_DIRECT $sDiskName"
 	command $sCommand >> $sFileReportTemp
 	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with the $FWBM_SC_HDPARM test"
		return $FWBM_SC_ERROR
	fi

	#Save the results
	fwBenchmarkScript_Save_Hdparm_Direct "$sFileReport" "$sFileReportTemp" "$sCommand" "$sDiskName"
 	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_SUCCESS" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The report was saved as $sFileReport."
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error saving the file."
		return $FWBM_SC_ERROR
	fi

	#Try to delete the temporal file. Not critical..
	rm -f $sFileReportTemp

	return $FWBM_SC_SUCCESS
}

function fwBenchmarkScipt_Hdparm_Core_Cache()
{
	local sBaseFileName=$1
	local sDiskName=$2

	local sName="CacheRead"

	#Report file name
	local iDate="$(date +%Y_%m_%d-%H_%M_%S)"
	local sFileReport=$iDate"-"$FWBM_SC_HDPARM"-"$sName".txt"
	local sFileReport=$sBaseFileName"/"$sFileReport
	local sFileReportTemp=$(mktemp)

	#Running test
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "Running the $FWBM_SC_HDPARM, Cache read test on $sDiskName."
	local sCommand="sudo $FWBM_SC_HDPARM -$FWBM_SC_HDPARM_CACHE $sDiskName"
 	command $sCommand >> $sFileReportTemp
 	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with the $FWBM_SC_HDPARM test"
		return $FWBM_SC_ERROR
	fi

	#Save the results
	fwBenchmarkScript_Save_Hdparm_Cache "$sFileReport" "$sFileReportTemp" "$sCommand" "$sDiskName"
 	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_SUCCESS" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The report was saved as $sFileReport."
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error saving the file."
		return $FWBM_SC_ERROR
	fi

	#Try to delete the temporal file. Not critical..
	rm -f $sFileReportTemp

	return $FWBM_SC_SUCCESS
}

