#!/bin/sh

# Sysbench information about how to use and the the significance of the results:		https://wiki.gentoo.org/wiki/Sysbench

function fwBenchmarkScript_Save_Sysbench_Cpu()
{
	local sFileReport="$1"
	local sFileReportTemp="$2"

	local sCommand="$3"

	local iMaxRequest="$4"
	local iMaxPrime="$5"
	local iCpuNumThreads="$6"


	#Save the report
	local sBenchmarkTitle="$FWBM_SC_SYSBENCH, test $FWBM_SC_SYSBENCH_CPU"
	echo $FWBM_SC_NEWSECTION > $sFileReport 	#Insert a new section in a EMPTY file (Only one symbol >)
	fwBenchmarkScript_UtilTests_GetReportCommon "$sBenchmarkTitle" "$sCommand" "$sFileReport"

echo "Test Description:
When running with the CPU workload,
sysbench will verify prime numbers by doing standard division
of the number by all numbers between 2 and the square root of the number.
If any number gives a remainder of 0, the next number is calculated." >> $sFileReport

	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Parameters:" >> $sFileReport
	echo "Iterations: $iMaxRequest" >> $sFileReport
	echo "Calculation of primes up value: $iMaxPrime" >> $sFileReport
	echo "Threads number: $iCpuNumThreads" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Statistics:" >> $sFileReport
	fwBenchmarkScript_Util_ReadFiles $sFileReportTemp		#results the report in an array, dsREADFILELINES[]

	local iNumberLinesFileReportTemp=${#dsREADFILELINES[@]}
	local iIter="0"
	local iTotalTimeCpuTest="0"
	local iTotalTimeEventCpuTest="0"
	while [ $iIter -le $iNumberLinesFileReportTemp ]
	do
		local iTotalTimeCpuTestTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'total time:')"
		if [ "$iTotalTimeCpuTestTemp" != "" ]; then
			local iTotalTimeCpuTest=$iTotalTimeCpuTestTemp
		fi

		local iTotalTimeEventCpuTestTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'total time taken by event execution:')"
		if [ "$iTotalTimeEventCpuTestTemp" != "" ]; then
			local iTotalTimeEventCpuTest=$iTotalTimeEventCpuTestTemp
		fi

		local iIter=$[$iIter+1]
	done

	#Show some parameters to metric
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$iTotalTimeCpuTest"
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$iTotalTimeEventCpuTest"

	echo "$iTotalTimeCpuTest" >> $sFileReport
	echo "$iTotalTimeEventCpuTest" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport
}

function fwBenchmarkScript_Save_Sysbench_Fileio()
{
	local sFileReport="$1"
	local sFileReportTemp="$2"

	local sCommand="$3"

	local sTotalSize="$4"
	local iMaxTime="$5"
	local sModeIo="$6"


	#Save the report
	local sBenchmarkTitle="$FWBM_SC_SYSBENCH, test $FWBM_SC_SYSBENCH_FILEIO"
	echo $FWBM_SC_NEWSECTION > $sFileReport 	#Insert a new section in a EMPTY file (Only one symbol >)
	fwBenchmarkScript_UtilTests_GetReportCommon "$sBenchmarkTitle" "$sCommand" "$sFileReport"

echo "Test Description:
This test mode can be used to produce various kinds of file I/O
workloads. At the prepare stage SysBench creates a specified number
of files with a specified total size, then at the run stage, each
thread performs specified I/O operations on this set of files." >> $sFileReport

	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Parameters:" >> $sFileReport
	echo "Total size of files: $sTotalSize" >> $sFileReport
	echo "Duration of the test: $iMaxTime seconds" >> $sFileReport
	echo "Operation with the files: $sModeIo (random read and write files)" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Statistics:" >> $sFileReport
	fwBenchmarkScript_Util_ReadFiles $sFileReportTemp		#results the report in an array, dsREADFILELINES[]

	local iNumberLinesFileReportTemp=${#dsREADFILELINES[@]}
	local iIter="0"

	local sOperationsPerformedNumFiles=""
	local sOperationsPerformedSizeFiles=""

	while [ $iIter -le $iNumberLinesFileReportTemp ]
	do
		local sOperationsPerformedNumFilesTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'Operations performed')"
		if [ "$sOperationsPerformedNumFilesTemp" != "" ]; then
			local sOperationsPerformedNumFiles=$sOperationsPerformedNumFilesTemp
		fi

		local sOperationsPerformedSizeFilesTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'Total transferred')"
		if [ "$sOperationsPerformedSizeFilesTemp" != "" ]; then
			local sOperationsPerformedSizeFiles=$sOperationsPerformedSizeFilesTemp
		fi

		local iIter=$[$iIter+1]
	done

	#Show some parameters to metric
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$sOperationsPerformedNumFiles"
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$sOperationsPerformedSizeFiles"

	echo "$sOperationsPerformedNumFiles" >> $sFileReport
	echo "$sOperationsPerformedSizeFiles" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport
}

function fwBenchmarkScript_Save_Sysbench_Threads()
{
	local sFileReport="$1"
	local sFileReportTemp="$2"

	local sCommand="$3"

	local iThreadYields="$4"
	local sThreadMaxTime="$5"


	#Save the report
	local sBenchmarkTitle="$FWBM_SC_SYSBENCH, test $FWBM_SC_SYSBENCH_THREADS"
	echo $FWBM_SC_NEWSECTION > $sFileReport 	#Insert a new section in a EMPTY file (Only one symbol >)
	fwBenchmarkScript_UtilTests_GetReportCommon "$sBenchmarkTitle" "$sCommand" "$sFileReport"

echo "Test Description:
With the threads workload, each worker thread will be allocated a mutex 
(a sort of lock) and will, for each execution, loop a number of times
(documented as the number of yields) in which it takes the lock, yields
(meaning it asks the scheduler to stop itself from running and put it back and
the end of the runqueue) and then, when it is scheduled again for execution, unlock." >> $sFileReport

	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Parameters:" >> $sFileReport
	echo "Number of lock/yield/unlock loops to execute per each request: $iThreadYields" >> $sFileReport
	echo "Duration of the test: $sThreadMaxTime" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Statistics:" >> $sFileReport
	fwBenchmarkScript_Util_ReadFiles $sFileReportTemp		#results the report in an array, dsREADFILELINES[]

	local iNumberLinesFileReportTemp=${#dsREADFILELINES[@]}
	local iIter="0"

	local sMin=""
	local sAvg=""
	local sMax=""
	local sApprox95=""

	while [ $iIter -le $iNumberLinesFileReportTemp ]
	do
		local sMinTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'min:')"
		if [ "$sMinTemp" != "" ]; then
			local sMin=$sMinTemp
		fi

		local sAvgTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'avg:')"
		if [ "$sAvgTemp" != "" ]; then
			local sAvg=$sAvgTemp
		fi

		local sMaxTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'max:')"
		if [ "$sMaxTemp" != "" ]; then
			local sMax=$sMaxTemp
		fi

		local sApprox95Temp="$(echo ${dsREADFILELINES[$iIter]} | grep 'approx.')"
		if [ "$sApprox95Temp" != "" ]; then
			local sApprox95=$sApprox95Temp
		fi

		local iIter=$[$iIter+1]
	done

	#Show some parameters to metric
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "per-request statistics:"
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$sMin"
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$sAvg"
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$sMax"

	echo "per-request statistics:" >> $sFileReport
	echo "- $sMin" >> $sFileReport
	echo "- $sAvg" >> $sFileReport
	echo "- $sMax" >> $sFileReport
	echo "- $sApprox95" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport
}


function fwBenchmarkScript_Save_Sysbench_Mutex()
{
	local sFileReport="$1"
	local sFileReportTemp="$2"

	local sCommand="$3"

	local iMutexNumThreads="$4"


	#Save the report
	local sBenchmarkTitle="$FWBM_SC_SYSBENCH, test $FWBM_SC_SYSBENCH_MUTEX"
	echo $FWBM_SC_NEWSECTION > $sFileReport 	#Insert a new section in a EMPTY file (Only one symbol >)
	fwBenchmarkScript_UtilTests_GetReportCommon "$sBenchmarkTitle" "$sCommand" "$sFileReport"

echo "Test Description:
The sysbench application will run a single request per thread.
This request will first put some stress on the CPU (using a simple incremental loop)
after which it takes a random mutex (lock), increments a global variable and releases
the lock again." >> $sFileReport

	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Parameters:" >> $sFileReport
	echo "Threads nummber: $iMutexNumThreads" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Statistics:" >> $sFileReport
	fwBenchmarkScript_Util_ReadFiles $sFileReportTemp		#results the report in an array, dsREADFILELINES[]

	local iNumberLinesFileReportTemp=${#dsREADFILELINES[@]}
	local iIter="0"

	local iTotalTime="0"
	local iTotalTimeEvent="0"

	while [ $iIter -le $iNumberLinesFileReportTemp ]
	do
		local iTotalTimeTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'total time:')"
		if [ "$iTotalTimeTemp" != "" ]; then
			local iTotalTime=$iTotalTimeTemp
		fi

		local iTotalTimeEventTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'total time taken by event execution:')"
		if [ "$iTotalTimeEventTemp" != "" ]; then
			local iTotalTimeEvent=$iTotalTimeEventTemp
		fi

		local iIter=$[$iIter+1]
	done

	#Show some parameters to metric
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$iTotalTime"
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$iTotalTimeEvent"

	echo "$iTotalTime" >> $sFileReport
	echo "$iTotalTimeEvent" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport
}

function fwBenchmarkScript_Save_Sysbench_Memory()
{
	local sFileReport="$1"
	local sFileReportTemp="$2"

	local sCommand="$3"

	local sMemoryTotalSize="$4"
	local iMemoryNumThreads="$5"


	#Save the report
	local sBenchmarkTitle="$FWBM_SC_SYSBENCH, test $FWBM_SC_SYSBENCH_MEMORY"
	echo $FWBM_SC_NEWSECTION > $sFileReport 	#Insert a new section in a EMPTY file (Only one symbol >)
	fwBenchmarkScript_UtilTests_GetReportCommon "$sBenchmarkTitle" "$sCommand" "$sFileReport"

echo "Test Description:
The benchmark application will allocate a memory buffer and then read or write from it,
each time for the size of a pointer (so 32bit or 64bit), and each execution until the
total buffer size has been read from or written to.
This is then repeated until the provided volume (--memory-total-size) is reached. " >> $sFileReport

	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Parameters:" >> $sFileReport
	echo "Memory transfer size: $sMemoryTotalSize" >> $sFileReport
	echo "Threads number: $iMemoryNumThreads" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Statistics:" >> $sFileReport
	fwBenchmarkScript_Util_ReadFiles $sFileReportTemp		#results the report in an array, dsREADFILELINES[]

	local iNumberLinesFileReportTemp=${#dsREADFILELINES[@]}
	local iIter="0"

	local sOperationsPerformedNumFiles=""
	local sOperationsPerformedSizeFiles=""

	while [ $iIter -le $iNumberLinesFileReportTemp ]
	do
		local sOperationsPerformedNumFilesTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'Operations performed')"
		if [ "$sOperationsPerformedNumFilesTemp" != "" ]; then
			local sOperationsPerformedNumFiles=$sOperationsPerformedNumFilesTemp
		fi

		local sOperationsPerformedSizeFilesTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'transferred')"
		if [ "$sOperationsPerformedSizeFilesTemp" != "" ]; then
			local sOperationsPerformedSizeFiles=$sOperationsPerformedSizeFilesTemp
		fi

		local iIter=$[$iIter+1]
	done

	#Show some parameters to metric
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$sOperationsPerformedNumFiles"
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$sOperationsPerformedSizeFiles"

	echo "$sOperationsPerformedNumFiles" >> $sFileReport
	echo "$sOperationsPerformedSizeFiles" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport
}

function fwBenchmarkScript_Save_Hdparm_Direct()
{
	local sFileReport="$1"
	local sFileReportTemp="$2"

	local sCommand="$3"

	local sDiskName="$4"


	#Save the report
	local sBenchmarkTitle="$FWBM_SC_HDPARM, Direct device read test"
	echo $FWBM_SC_NEWSECTION > $sFileReport 	#Insert a new section in a EMPTY file (Only one symbol >)
	fwBenchmarkScript_UtilTests_GetReportCommon "$sBenchmarkTitle" "$sCommand" "$sFileReport"

echo "Test Description:
Perform  timings  of  device reads for benchmark and comparison purposes.
This displays the  speed of reading through the buffer cache to the disk without 
any prior caching of data.  This measurement is an indication of how 
fast the drive can sustain sequential data reads under Linux, without 
any filesystem overhead." >> $sFileReport

	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Parameters:" >> $sFileReport
	echo "Disk on which the test is performed: $sDiskName" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Statistics:" >> $sFileReport
	fwBenchmarkScript_Util_ReadFiles $sFileReportTemp		#results the report in an array, dsREADFILELINES[]

	local iNumberLinesFileReportTemp=${#dsREADFILELINES[@]}
	local iIter="0"

	local sTiming=""

	while [ $iIter -le $iNumberLinesFileReportTemp ]
	do
		local sTimingTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'reads:')"
		if [ "$sTimingTemp" != "" ]; then
			local sTiming=$sTimingTemp
		fi

		local iIter=$[$iIter+1]
	done

	#Show some parameters to metric
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$sTiming"

	echo "$sTiming" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport
}

function fwBenchmarkScript_Save_Hdparm_Cache()
{
	local sFileReport="$1"
	local sFileReportTemp="$2"

	local sCommand="$3"

	local sDiskName="$4"


	#Save the report
	local sBenchmarkTitle="$FWBM_SC_HDPARM, Cache read test"
	echo $FWBM_SC_NEWSECTION > $sFileReport 	#Insert a new section in a EMPTY file (Only one symbol >)
	fwBenchmarkScript_UtilTests_GetReportCommon "$sBenchmarkTitle" "$sCommand" "$sFileReport"

echo "Test Description:
Perform timings of cache reads for benchmark and comparison purposes.
This displays the speed of reading directly from the Linux buffer cache without 
disk access.  This measurement is essentially an indication of the
throughput of the processor, cache, and memory of the system under test." >> $sFileReport

	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Parameters:" >> $sFileReport
	echo "Disk on which the test is performed: $sDiskName" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport

	echo "Statistics:" >> $sFileReport
	fwBenchmarkScript_Util_ReadFiles $sFileReportTemp		#results the report in an array, dsREADFILELINES[]

	local iNumberLinesFileReportTemp=${#dsREADFILELINES[@]}
	local iIter="0"

	local sTiming=""

	while [ $iIter -le $iNumberLinesFileReportTemp ]
	do
		local sTimingTemp="$(echo ${dsREADFILELINES[$iIter]} | grep 'reads:')"
		if [ "$sTimingTemp" != "" ]; then
			local sTiming=$sTimingTemp
		fi

		local iIter=$[$iIter+1]
	done

	#Show some parameters to metric
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "$sTiming"

	echo "$sTiming" >> $sFileReport
	echo $FWBM_SC_NEWSECTION >> $sFileReport
}