#!/bin/sh

function fwBenchmarkScript_Sysbench_Launch()
{
	local sBaseFileName=$1
	local sTestModeName=$2

	echo $FWBM_SC_NEWSECTION
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Starting the $FWBM_SC_SYSBENCH test"

	#Check if the benchmark is installed
	fwBenchmarkScript_UtilTests_Init $FWBM_SC_SYSBENCH
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Stopping the benchmark due to error."
		exit $FWBM_SC_ERROR
	fi

	#Run the tests
	case "$sTestModeName" in
		$FWBM_SC_SYSBENCH_CPU)
 			fwBenchmarkScript_SysBench_Run_Cpu $sBaseFileName
 			local iErr=$?
			if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
				return $FWBM_SC_ERROR
			fi
			;;

		$FWBM_SC_SYSBENCH_FILEIO)
 			fwBenchmarkScript_SysBench_Run_Fileio $sBaseFileName
 			local iErr=$?
			if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
				return $FWBM_SC_ERROR
			fi
			;;

		$FWBM_SC_SYSBENCH_THREADS)
 			fwBenchmarkScript_SysBench_Run_Threads $sBaseFileName
 			local iErr=$?
			if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
				return $FWBM_SC_ERROR
			fi
			;;

		$FWBM_SC_SYSBENCH_MUTEX)
 			fwBenchmarkScript_SysBench_Run_Mutex $sBaseFileName
 			local iErr=$?
			if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
				return $FWBM_SC_ERROR
			fi
			;;

		$FWBM_SC_SYSBENCH_MEMORY)
 			fwBenchmarkScript_SysBench_Run_Memory $sBaseFileName
 			local iErr=$?
			if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
				return $FWBM_SC_ERROR
			fi
			;;

		*)	fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "The test $FWBM_SC_SYSBENCH is unknow"
 			return $FWBM_SC_ERROR
			;;
	esac

	return $FWBM_SC_SUCCESS
}

function fwBenchmarkScript_SysBench_Run_Cpu()
{
	local sBaseFileName=$1

	#Report file name
	local iDate="$(date +%Y_%m_%d-%H_%M_%S)"
	local sFileReport=$iDate"-"$FWBM_SC_SYSBENCH"-"$FWBM_SC_SYSBENCH_CPU".txt"
	local sFileReport=$sBaseFileName"/"$sFileReport
	local sFileReportTemp=$(mktemp)

	#var FWBM_SC_SYSBENCH_CPU
	local iMaxRequest=10000
	local iMaxPrime=20000
	local iCpuNumThreads=1

	#Running the test
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "Running the $FWBM_SC_SYSBENCH, $FWBM_SC_SYSBENCH_CPU test."
	local sCommand="sysbench --test=$FWBM_SC_SYSBENCH_CPU --max-requests=$iMaxRequest --cpu-max-prime=$iMaxPrime --num-threads=$iCpuNumThreads run"
	command $sCommand >> $sFileReportTemp

 	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_SUCCESS" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "$FWBM_SC_SYSBENCH_CPU test has been executed correctly."
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with the $FWBM_SC_SYSBENCH_CPU test."
		return $FWBM_SC_ERROR
	fi

	#Save the results
	fwBenchmarkScript_Save_Sysbench_Cpu "$sFileReport" "$sFileReportTemp" "$sCommand" "$iMaxRequest" "$iMaxPrime" "$iCpuNumThreads"
 	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_SUCCESS" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The report was saved as $sFileReport."
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error saving the file."
		return $FWBM_SC_ERROR
	fi

	#Try to delete the temporal file. Not critical..
	rm -f $sFileReportTemp

	return $FWBM_SC_SUCCESS
}

function fwBenchmarkScript_SysBench_Run_Fileio()
{
	local sBaseFileName=$1

	#Report file name
	local iDate="$(date +%Y_%m_%d-%H_%M_%S)"
	local sFileReport=$iDate"-"$FWBM_SC_SYSBENCH"-"$FWBM_SC_SYSBENCH_FILEIO".txt"
	local sFileReport=$sBaseFileName"/"$sFileReport
	local sFileReportTemp=$(mktemp)

	#var FWBM_SC_SYSBENCH_FILEIO
	local sTotalSize="1G"
	local iMaxTime=30
	local sModeIo="rndrw"	#Combined random read/write

	#Running the test
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "Running the $FWBM_SC_SYSBENCH, $FWBM_SC_SYSBENCH_FILEIO test."

	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Creating files for the $FWBM_SC_SYSBENCH_FILEIO test in the same directory..."
	command sysbench --test=$FWBM_SC_SYSBENCH_FILEIO --file-total-size=$sTotalSize prepare >/dev/null
	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with the $FWBM_SC_SYSBENCH_FILEIO test, prepare function"
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Probably, it is not possible to create or read/write in this location"
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "(Network drive, or without permission). Try running the Benchmark from tmp folder e.g.: cd /tmp && sudo ./Projects/fwbenchmarkScriptStd.sh"
		fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Removing test files..."
		command sysbench --test=fileio --file-total-size=$sTotalSize cleanup >/dev/null
		return $FWBM_SC_ERROR
	fi

	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Running the test..."
 	local sCommand="sysbench --test=$FWBM_SC_SYSBENCH_FILEIO  --file-total-size=$sTotalSize --file-test-mode=$sModeIo --max-time=$iMaxTime --max-requests=0 --file-extra-flags=direct run"
	command $sCommand >> $sFileReportTemp
	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with the $FWBM_SC_SYSBENCH_FILEIO test, run function"
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Probably, it is not possible to create or read/write in this location"
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "(Network drive, or without permission). Try running the Benchmark from tmp folder  e.g.: cd /tmp && sudo ./Projects/fwbenchmarkScriptStd.sh"
		fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Removing test files..."
		command sysbench --test=fileio --file-total-size=$sTotalSize cleanup >/dev/null
		return $FWBM_SC_ERROR
	fi
	
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "Removing test files..."
	command sysbench --test=fileio --file-total-size=$sTotalSize cleanup >/dev/null
	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with the $FWBM_SC_SYSBENCH_FILEIO test, cleanup function"
		return $FWBM_SC_ERROR
	fi

	#Save the report
	fwBenchmarkScript_Save_Sysbench_Fileio "$sFileReport" "$sFileReportTemp" "$sCommand" "$sTotalSize" "$iMaxTime" "$sModeIo"
 	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_SUCCESS" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The report was saved as $sFileReport."
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error saving the file."
		return $FWBM_SC_ERROR
	fi

	#Try to delete the temporal file. Not critical..
	rm -f $sFileReportTemp
	
	return $FWBM_SC_SUCCESS
}

function fwBenchmarkScript_SysBench_Run_Threads()
{
	local sBaseFileName=$1

	#Report file name
	local iDate="$(date +%Y_%m_%d-%H_%M_%S)"
	local sFileReport=$iDate"-"$FWBM_SC_SYSBENCH"-"$FWBM_SC_SYSBENCH_THREADS".txt"
	local sFileReport=$sBaseFileName"/"$sFileReport
	local sFileReportTemp=$(mktemp)

	#var FWBM_SC_SYSBENCH_THREADS
	local iThreadYields=10000
	local sThreadMaxTime="20s"

	#Running the test
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "Running the $FWBM_SC_SYSBENCH, $FWBM_SC_SYSBENCH_THREADS test."
	local sCommand="sysbench --test=$FWBM_SC_SYSBENCH_THREADS --thread-locks=1 --thread-yields=$iThreadYields --max-time=$sThreadMaxTime run"
	command $sCommand >> $sFileReportTemp
	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with the $FWBM_SC_SYSBENCH_THREADS test"
		return $FWBM_SC_ERROR
	fi

	#Save the report
	fwBenchmarkScript_Save_Sysbench_Threads "$sFileReport" "$sFileReportTemp" "$sCommand" "$iThreadYields" "$sThreadMaxTime"
 	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_SUCCESS" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The report was saved as $sFileReport."
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error saving the file."
		return $FWBM_SC_ERROR
	fi

	#Try to delete the temporal file. Not critical..
	rm -f $sFileReportTemp

	return $FWBM_SC_SUCCESS
}

function fwBenchmarkScript_SysBench_Run_Mutex()
{
	local sBaseFileName=$1

	#Report file name
	local iDate="$(date +%Y_%m_%d-%H_%M_%S)"
	local sFileReport=$iDate"-"$FWBM_SC_SYSBENCH"-"$FWBM_SC_SYSBENCH_MUTEX".txt"
	local sFileReport=$sBaseFileName"/"$sFileReport
	local sFileReportTemp=$(mktemp)

	#var FWBM_SC_SYSBENCH_MUTEX
	local iMutexNumThreads=64

	#Running the test
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "Running the $FWBM_SC_SYSBENCH, $FWBM_SC_SYSBENCH_MUTEX test."
	local sCommand="sysbench --test=$FWBM_SC_SYSBENCH_MUTEX --num-threads=$iMutexNumThreads run"
	command $sCommand >> $sFileReportTemp
	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with the $FWBM_SC_SYSBENCH_MUTEX test"
		return $FWBM_SC_ERROR
	fi

	#Save the report
	fwBenchmarkScript_Save_Sysbench_Mutex "$sFileReport" "$sFileReportTemp" "$sCommand" "$iMutexNumThreads"
 	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_SUCCESS" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The report was saved as $sFileReport."
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error saving the file."
		return $FWBM_SC_ERROR
	fi

	#Try to delete the temporal file. Not critical..
	rm -f $sFileReportTemp

	return $FWBM_SC_SUCCESS
}

function fwBenchmarkScript_SysBench_Run_Memory()
{
	local sBaseFileName=$1

	#Report file name
	local iDate="$(date +%Y_%m_%d-%H_%M_%S)"
	local sFileReport=$iDate"-"$FWBM_SC_SYSBENCH"-"$FWBM_SC_SYSBENCH_MEMORY".txt"
	local sFileReport=$sBaseFileName"/"$sFileReport
	local sFileReportTemp=$(mktemp)

	#var FWBM_SC_SYSBENCH_MEMORY
	local sMemoryTotalSize="4G"
	local iMemoryNumThreads=1

	#Running the test
	fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_HIGH_LOG "Running the $FWBM_SC_SYSBENCH, $FWBM_SC_SYSBENCH_MEMORY test."
	local sCommand="sysbench --test=$FWBM_SC_SYSBENCH_MEMORY --num-threads=$iMemoryNumThreads --memory-total-size=$sMemoryTotalSize run"
	command $sCommand >> $sFileReportTemp
	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_ERROR" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error with the $FWBM_SC_SYSBENCH_MEMORY test"
		return $FWBM_SC_ERROR
	fi

	#Save the report
	fwBenchmarkScript_Save_Sysbench_Memory "$sFileReport" "$sFileReportTemp" "$sCommand" "$sMemoryTotalSize" "$iMemoryNumThreads"
 	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_SUCCESS" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The report was saved as $sFileReport."
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Error saving the file."
		return $FWBM_SC_ERROR
	fi

	#Try to delete the temporal file. Not critical..
	rm -f $sFileReportTemp

	return $FWBM_SC_SUCCESS
}

