#!/bin/sh

declare -a dsREADFILELINES

#This function check if the script has been launched with administrator privileges.
#Return FWBM_SC_ERROR (1) if there is not permits administrator and returns FWBM_SC_SUCCESS(0) it there is.
function fwBenchmarkScript_Util_AdminPermission()
{
	if [ $(id -u) != "0" ]; then
	    fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "You are '$USER' user, you don't have permission administrator"
	    return $FWBM_SC_ERROR
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "You are '$USER' user, you have permission administrator"
		return $FWBM_SC_SUCCESS
	fi

	return $FWBM_SC_SUCCESS
}

#This function write in the terminal a logger with the date, type of information and the comment, with colors.
function fwBenchmarkScript_Util_Logs()
{
	local iLogLevel=$1
	local sMessage=$2

	local iDate="$(date +%Y.%m.%d\ %H:%M:%S)"

	case "$iLogLevel" in

	"$FWBM_SC_ERROR_LOG")		local sLogLevel="ERROR"
 								local sText="$iDate [$sLogLevel]\t $sMessage"
 								fwBenchmarkScript_Util_EchoColor "$sText" "Red"
 								;;

	"$FWBM_SC_WARNING_LOG")		local sLogLevel="WARNING"
 								local sText="$iDate [$sLogLevel]\t $sMessage"
 								fwBenchmarkScript_Util_EchoColor "$sText" "Yellow"
								;;

	"$FWBM_SC_INFO_LOG")		local sLogLevel="INFO"
 								local sText="$iDate [$sLogLevel]\t $sMessage"
 								fwBenchmarkScript_Util_EchoColor "$sText" "Normal"
								;;

	"$FWBM_SC_INFO_HIGH_LOG")	local sLogLevel="INFO"
 								local sText="$iDate [$sLogLevel]\t $sMessage"
 								fwBenchmarkScript_Util_EchoColor "$sText" "Cyan"
								;;

	"$FWBM_SC_SUCCESS_LOG")		local sLogLevel="SUCCESS"
 								local sText="$iDate [$sLogLevel]\t $sMessage"
 								fwBenchmarkScript_Util_EchoColor "$sText" "Green"
								;;
	*) ;;
	esac
}

#This function helps to the Logs function for add the colour
function fwBenchmarkScript_Util_EchoColor()
{
	local sText=$1
	local sColor=$2

	local Black="\033[0;30m"        # Black
	local Red="\033[0;31m"          # Red
	local Green="\033[0;32m"        # Green
	local Yellow="\033[0;33m"       # Yellow
	local Blue="\033[0;34m"         # Blue
	local Purple="\033[0;35m"       # Purple
	local Cyan="\033[0;36m"         # Cyan
	local White="\033[0;37m"        # White
	local Normal="\e[0m"

	case "$sColor" in

	"Green")	echo -e $Green$sText$Normal
 				;;
	"Red")		echo -e $Red$sText$Normal
				;;
	"Yellow")	echo -e $Yellow$sText$Normal
				;;
	"Normal")	echo -e $sText$Normal
				;;
	"Cyan")		echo -e $Cyan$sText$Normal
				;;
	*) echo -e "$Normal"
	   ;;
	esac

	return $FWBM_SC_SUCCESS
}

#This function sends an email, using the "sendEmail" library located in the WinCCOA binaries.
function fwBenchmarkScript_Util_SendEmail()
{
	local sTo=$1
	local sSubject=$2
	local sContent=$3
	local sFileAttach=$4
  
	local smtpServer="cernmx.cern.ch"
	local sFrom="fwBenchmark@cern.ch"
	local sBody=$sContent

	local sCurrentTime="$(date +%Y.%m.%d\ %H:%M:%S)"
	local sHostname="$(hostname)"
	local sBody=$sBody"Email sent from $sHostname at $sCurrentTime."
	local sBody=$sBody"This is an automatically generated email. Please do not reply."

	local emailCommand="$FWBM_SC_WCCOA_DIR/bin/sendEmail -f $sFrom -t $sTo -u $sSubject -m $sBody -s $smtpServer"

	if [ -a $sFileAttach ];then
		local emailCommand="$emailCommand -a $sFileAttach"
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "File doesn't exists. It is not possible send the results by e-mail"
		return $FWBM_SC_ERROR
	fi

	command $emailCommand > /dev/null 2>&1
	local iErr=$?
	if [ "$iErr" -eq "$FWBM_SC_SUCCESS" ]; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "The report was sent to $sTo"
		return $FWBM_SC_SUCCESS
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "Unable to send the e-mail"
		return $FWBM_SC_ERROR
	fi
}

#This function read line per line a file, and returns the information in a global variable called "dsREADFILELINES"
function fwBenchmarkScript_Util_ReadFiles()
{
	unset dsREADFILELINES	#be careful, it's a global variable!

	local i=0
	while IFS='' read -r line || [[ -n "$line" ]]; do
		#echo "$i Text read from file: $line"
		dsREADFILELINES[i]="$line"
		let "i++"
	done < "$1"

	#other way:	dsREADFILELINES=(`cat "$1" `)

	# echo ${dsREADFILELINES[0]}	#First element
	# echo ${dsREADFILELINES[1]}	#Second

	# echo ${dsREADFILELINES[@]}	#Print all array
	# echo ${#dsREADFILELINES[@]}	#Number elements
}
