#!/bin/sh

function fwBenchmarkScript_UtilTests_DisplayHelp()
{
    echo "Usage: $0 [option]... [name]..." >&2
    echo "e.g.   sudo $0 -d /home/Results -m alejandro.herrero.de.campos@cern.ch"
    echo "e.g. extended:"
    echo "cd /tmp/ && sudo /afs/cern.ch/user/a/aherrero/private/Projects/fwBenchmark/Source/fwBenchmark/fwBenchmarkScriptStd.sh -d /afs/cern.ch/user/a/aherrero/DataResults -m alejandro.herrero.de.campos@cern.ch"
    echo
    echo "   -d, --directory         Type a directory to save the results in another path."
    echo "   -m, --mail              Send the results by email."
    echo "   -h, --help              Show this help."
    echo
}

function fwBenchmarkScript_UtilTests_Init()
{
	local sBenchMark=$1
	## Check if the BENCHMARKNAME is installed. If not, and you are admin, install.
	if command -v $sBenchMark > /dev/null 2>&1; then
		fwBenchmarkScript_Util_Logs $FWBM_SC_INFO_LOG "$sBenchMark is installed."
	else
		fwBenchmarkScript_Util_Logs $FWBM_SC_WARNING_LOG "$sBenchMark is not installed"
		fwBenchmarkScript_Util_AdminPermission
		local iAdminPermission=$?	#Save the return value. If admin returns 0, else returns 1.
		if [ "$iAdminPermission" -eq "1" ]; then
			fwBenchmarkScript_Util_Logs $FWBM_SC_ERROR_LOG "You don't have permission administrator to install it"
		    return $FWBM_SC_ERROR
		else
			fwBenchmarkScript_Util_Logs $FWBM_SC_SUCCESS_LOG "You have permission. Installing $sBenchMark..."
			sudo yum install --enablerepo=epel -y $sBenchMark
		fi
	fi

	return $FWBM_SC_SUCCESS
}

function fwBenchmarkScript_UtilTests_GetReportCommon()
{
	local sBenchmarkTitle="$1"
	local sCommand="$2"
	local sFileName="$3"

	local iDate="$(date +%Y.%m.%d_%H:%M:%S)"

	echo "Date/Time of the report generation: $iDate" >> $sFileName
	echo $FWBM_SC_NEWSECTION >> $sFileName
	echo "Benchmark name: $sBenchmarkTitle" >> $sFileName
	echo "Command: $sCommand" >> $sFileName
	echo $FWBM_SC_NEWSECTION >> $sFileName

	#Machine Characteristics
	echo $FWBM_SC_NEWSECTION >> $sFileName
	local sHostname="$(hostname)"
	echo "Hostname: $sHostname" >> $sFileName

 	local sProcName="$(cat /proc/cpuinfo | grep name  | cut -d ':' -f2)"
	local iNumCore="$(grep -c ^processor /proc/cpuinfo)"
	echo "Number of cores: $iNumCore" >> $sFileName
	echo "Processor: $sProcName" >> $sFileName

	local sOperatingSystem="$(uname)"
	local sOperatingSystemDetails="$(uname -s -r -v -m -p -i -o)"
	echo "Operating system: $sOperatingSystem" >> $sFileName
	echo "Operating system details: $sOperatingSystemDetails" >> $sFileName
	echo $FWBM_SC_NEWSECTION >> $sFileName
}
