#!/usr/bin/python
import os
import urlparse
import time
import shutil
import signal
import sys
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapper import *

PYTHON_SCRIPT_PATH = os.path.dirname(os.path.realpath(__file__))		#"tmp/WinCCOAProject"
FWBENCHMARK_ROOT_PATH = PYTHON_SCRIPT_PATH[0:PYTHON_SCRIPT_PATH.rfind("Source")]
INSTALL_PATH = FWBENCHMARK_ROOT_PATH + "Test/projects_created"
FWINSTALLATION_URL = "https://edms.cern.ch/file/1085149/67/fwInstallation-7.2.6.zip"

def main(arguments):
	iErr = fwBenchmarkBootstrapper_Launch(arguments)
	sys.exit(iErr)

def signal_handler(signal, frame):
	print "\nYou stopped the benchmark uncomplete.\n"
	sContinueBenchmark = raw_input("Do you want to stop all the WinCC-OA projects? [y/N]\n")
	if sContinueBenchmark == "y" or sContinueBenchmark == "Y":
		print "Stopping the projects..."
		PVSSBootstrapper.stopAllProjects()
		sys.exit(1)
	else:
		sys.exit(1)


def fwBenchmarkBootstrapper_Launch(arguments):
	signal.signal(signal.SIGINT, signal_handler)		#ctrl + C
	signal.signal(signal.SIGTSTP, signal_handler)		#ctrl + Z

	sEmail = "";
	sProjectName = "";

	## Arguments
	if len(arguments) == 2:
		sProjectName = arguments[1]

	elif len(arguments) == 3:
		sProjectName = arguments[1]
		sEmail = arguments[2]
	else:
		print "Something wrong with arguments in python script."
		return 1

	print "The project called " + sProjectName + " will be created in the default directory: " + INSTALL_PATH

	## Check if the scripts is in the afs folder
	if PYTHON_SCRIPT_PATH.find("/afs/") != -1:
		print "The script is into the afs directory, which is a remote disk. It would be better act directly in the machine disk. (Copy the fwBenchmark component into a root folder)"
		sContinueBenchmark = raw_input("Do you want to continue? [y/N]")
		if sContinueBenchmark == "n" or sContinueBenchmark == "N":
			print "Stopping the fwBenchmark due afs folder..."
			sys.exit(1)
		elif sContinueBenchmark == "y" or sContinueBenchmark == "Y":
			print "Continue..."
		else:
			print "Unknow option " + sContinueBenchmark
			sys.exit(1)

	## Initialize
	PVSSBootstrapper.stopAllProjects()
	print "Deleting existing projects with the same name"
	if PVSSBootstrapper.isProjectRegistered(sProjectName):
		PVSSBootstrapper.deleteProject(sProjectName)
		time.sleep(0.2)
		if PVSSBootstrapper.isProjectRegistered(sProjectName):
			PVSSBootstrapper.unregisterProject(sProjectName)

	# PATH Operations
	sInstallProjectPath = INSTALL_PATH + "/" + sProjectName
	# If there is a path called sProjectName, remove all the tree
	if os.path.isdir(INSTALL_PATH):
		print "Removing path " + INSTALL_PATH
		shutil.rmtree(INSTALL_PATH)
	time.sleep(0.2)

	os.mkdir(INSTALL_PATH)
	time.sleep(0.2)
	if os.path.isdir(INSTALL_PATH) == False:
		print "Error creating the installation folder, " + INSTALL_PATH

	## Create
	print "Creating the project " + sProjectName
	PVSSBootstrapper.createProject(INSTALL_PATH, sProjectName)

	## fwInstallation tool
	print "Downloading the fwInstallation tool and extract"
	sFwInstallationToolDestination = os.path.join(sInstallProjectPath, os.path.basename(urlparse.urlsplit(FWINSTALLATION_URL).path))
	fwInstallation = ""

	if os.path.exists(sFwInstallationToolDestination):
		fwInstallation = sFwInstallationToolDestination
	else:
		fwInstallation = PVSSBootstrapper.grabURLFile(FWINSTALLATION_URL, sInstallProjectPath)

	PVSSBootstrapper.deployZip(sProjectName, fwInstallation)

	## Components: fwBenchmark
	print "Installing the fwBenchmark component"
	fwBenchmark = FWBENCHMARK_ROOT_PATH + "PVSS"

	sInstallComponentFolder = sInstallProjectPath + "/installed_components"
	os.mkdir(sInstallComponentFolder)
	time.sleep(0.5)
	if os.path.isdir(sInstallProjectPath) == False:
		print "Error creating the components folder, " + sInstallProjectPath
	PVSSBootstrapper.startProject(sProjectName)
	PVSSBootstrapper.setComponentInstallationFolder(sProjectName, sInstallComponentFolder)
	PVSSBootstrapper.installComponent(sProjectName, sources=fwBenchmark, components=['fwBenchmark.xml'])
	time.sleep(1)

	print "Component installed, restarting the project"
	PVSSBootstrapper.startProject(sProjectName, restart=True)

	## Launch the fwBenchmark control
	print "Running the fwBenchmark control with WinCCOA. It takes about 30 minutes, therefore, look at the Log Viewer to check progress."
	wccoa = WinCCOA.getDefaultInstance()
	wccoa.runManagerInProject("ctrl", sProjectName, "fwBenchmark/fwBenchmarkStd.ctl", "-mail " + sEmail)

	return 0

if __name__ == '__main__':
	# Call the function, providing the parameters from the bash script
	main(sys.argv)